--#****************************************************************************
--#**
--#**  New File:   /mods/gilbotsimprovements/autotoggle.lua
--#**  Modded by:  Gilbot-X
--#**
--#**  Summary  :  
--#**     This is the code for Mass Fabricator units and 
--#**     and all intel units that power down automatically 
--#**     when the player's economy is low on energy.
--#**
--#**
--#****************************************************************************

--# This effects what is output to the log.
--# Switch it to false if you are not debugging.
local debugAutoToggle = true



--# This function is called to create the class 
--# so this class can add its 
--# code to different base classes.
function MakeAutoToggleUnit(baseClassArg, powerDownToggleBitArg, 
                                          priorityCategoryArg)
    
    local settings = {
        PriorityCategory = priorityCategoryArg or nil, 
        PowerDownToggleName = nil,
        ToggleOnPosition = false,
        ToggleOffPosition = true,
    }
    
    --# This unit has shield so let ACU use Auto-toggle on it 
    if powerDownToggleBitArg == 0 then
        settings.PowerDownToggleName = 'RULEUTC_ShieldToggle'
        settings.ToggleOnPosition = true
        settings.ToggleOffPosition = false
        settings.PriorityCategory = priorityCategoryArg or 5
      
    --# This unit has jammer so let ACU use Auto-toggle on it       
    elseif powerDownToggleBitArg == 2 then
        settings.PowerDownToggleName = 'RULEUTC_JammingToggle'
        settings.PriorityCategory = priorityCategoryArg or 4

    --# This unit has intel so let ACU use Auto-toggle on it 
    elseif powerDownToggleBitArg == 3 then
        settings.PowerDownToggleName = 'RULEUTC_IntelToggle'
        settings.PriorityCategory = priorityCategoryArg or 3

    --# This unit has stealth so let ACU use Auto-toggle on it 
    elseif powerDownToggleBitArg == 5 then
        settings.PowerDownToggleName = 'RULEUTC_StealthToggle'
        settings.PriorityCategory = priorityCategoryArg or 4

    --# This unit has ? so let ACU use Auto-toggle on it 
    elseif powerDownToggleBitArg == 4 then
        settings.PowerDownToggleName = 'RULEUTC_ProductionToggle'
        settings.PriorityCategory = priorityCategoryArg or 1
    
    --# This unit has ? so let ACU use Auto-toggle on it 
    elseif powerDownToggleBitArg == 6 then
        settings.PowerDownToggleName = 'RULEUTC_GenericToggle'
        settings.PriorityCategory = priorityCategoryArg or 2
      
    --# This unit has ? so let ACU use Auto-toggle on it     
    elseif powerDownToggleBitArg == 7 then
        settings.PowerDownToggleName = 'RULEUTC_SpecialToggle'
        settings.PriorityCategory = priorityCategoryArg or 2

    --# This unit has cloak so let ACU use Auto-toggle on it 
    elseif powerDownToggleBitArg == 8 then
        settings.PowerDownToggleName = 'RULEUTC_CloakToggle'
        settings.PriorityCategory = priorityCategoryArg or 4
    else
        --# Inappropriate argument.  Programming error in third party unit mod.
        WARN("Gilbot-X's Modpack: Bad value for powerDownToggleBitArg in Autotoggle.lua")
        return baseClassArg
    end
    
    
local resultClass = Class(baseClassArg) {

    --# These are constants specific to the class being changed.
    --# They are meant to be shared but not modified by objects of the classes.
    PowerDownToggleBit = powerDownToggleBitArg,
    PowerDownToggleName = settings.PowerDownToggleName,
    PriorityCategory = settings.PriorityCategory,
    ToggleOnPosition = settings.ToggleOnPosition,
    ToggleOffPosition = settings.ToggleOffPosition,
    
    
    --# This constant shared variable provides a convenient means 
    --# of testing any object to see if it uses this class.
    IsAutoToggleUnit = true,
    
    
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  Overrided to call extra initialisation code.
    --#** 
    OnStopBeingBuilt = function(self,builder,layer)
        --# Call base class code first.
        baseClassArg.OnStopBeingBuilt(self,builder,layer)
        
        --# Call additional setup code.
        self:InitialiseAutoToggleUnit()
        
        --# If it is toggled off.. 
        if not self:GetScriptBit('RULEUTC_WeaponToggle') then
            --# Switch auto-toggle on
            self:SetScriptBit('RULEUTC_WeaponToggle', true)
        else 
            --# or just run the code that switches it on 
            self:OnScriptBit1Set() 
        end
    end,
    
    
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  Extra initialisation sets callbacks to tell ACU if we are destroyed.
    --#** 
    InitialiseAutoToggleUnit = function(self)
        --# Start switched on
        self.On = true
        --# Start permitted to be switched back on
        self.CanAutoToggleOnNow = true
        --# Record from BP how much energy this unit uses
        self.EnergyUsage = self:GetBlueprint().Economy.MaintenanceConsumptionPerSecondEnergy
        --# Record tech level now because we don't 
        --# want to have to query this on dead units
        self.TechLevel = self:GetMyTechLevel()
        --# This variable is for setting a custom position
        --# in the ordering in game by a player
        self.AutoTogglePositionsMoved = 0
        
        --# This next block is for debugging only.
        --# This can be delete wheh debugging is done.
        if debugAutoToggle then 
            self.DebugId = self:GetUnitId() .. '_' .. self:GetEntityId()
        end
        
        --# If the auto-toggle unit is no longer alive or working for us,
        --# then these callbacks update the ACU with this information.
        self:AddOnCapturedCallback(self.TellACU_NotToUseAutoToggleOnMe)
    end, 
    
    
    
    --# Create a Callback function to update our status
    TellACU_ToUseAutoToggleOnMe = function(self)
        --# ACU can keep track of networks while alive
        local myCommander = self:GetMyCommander()
        --# If commander is alive...
        if myCommander then
            myCommander:ReceiveACUMessage_RegisterAutoToggle(self)
            --# Inform that ACU was told
            self.CanUnregisterAutoToggle = true
            return true
        else
            --# Inform that ACU was not told
            return false
        end
    end,
    
    
    --# Create a Callback function to update our status
    TellACU_NotToUseAutoToggleOnMe = function(self)
        --# Don't let this be called when the autotoggle is already unregistered
        if self.CanUnregisterAutoToggle then
            --# ACU can keep track of networks while alive
            local myCommander = self:GetMyCommander()
            --# If commander is alive...
            if myCommander then
                myCommander:ReceiveACUMessage_UnregisterAutoToggle(self)
            end
            --# Don't let this be called when the autotoggle is already unregistered
            self.CanUnregisterAutoToggle = false
        end
    end,
    

    
    --# Create a Callback function to update our status
    TellACU_DecreaseMyPriority = function(self)
        --# ACU can keep track of networks while alive
        local myCommander = self:GetMyCommander()
        --# If commander is alive...
        if myCommander then
            myCommander:ReceiveACUMessage_DecreasePositionInPriorityTable(self)
        end
    end,
    
       
    --# Create a Callback function to update our status
    TellACU_IncreaseMyPriority = function(self)
        --# ACU can keep track of networks while alive
        local myCommander = self:GetMyCommander()
        --# If commander is alive...
        if myCommander then
            myCommander:ReceiveACUMessage_IncreasePositionInPriorityTable(self)
        end
    end,
    
 
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  This function makes sure that on/off state is valid
    --#*  when autotoggle calls for the toggle to be set. 
    --#*
    --#*  You can't call this from OnScriptBitSet or OnScriptBitClear
    --#*  because this function also tries to set the same script bit. 
    --#** 
    SwitchOff = function(self)
        if self:GetScriptBit(self.PowerDownToggleName) == self.ToggleOnPosition then 
            self:SetScriptBit(self.PowerDownToggleName, self.ToggleOffPosition) 
            self.On = false
            --# Signal to calling code that 
            --# a toggle change took place
            return true
        else
            --# Signal to calling code that 
            --# a toggle change did not take place
            return false
        end
    end,
    
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  This function makes sure that on/off state is valid
    --#*  when autotoggle calls for the toggle to be set.
    --#*
    --#*  You can't call this from OnScriptBitSet or OnScriptBitClear
    --#*  because this function also tries to set the same script bit. 
    --#** 
    SwitchOn = function(self)
        if self:GetScriptBit(self.PowerDownToggleName) == self.ToggleOffPosition then 
            self:SetScriptBit(self.PowerDownToggleName, self.ToggleOnPosition) 
            self.On = true
            --# Signal to calling code that 
            --# a toggle change took place
            return true
        else
            --# Signal to calling code that 
            --# a toggle change did not take place
            return false
        end 
    end,
    
    
    
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  These functions allow user to toggle whether or not 
    --#*  the ACU's auto-toggling can run on this unit. 
    --#** 
    OnScriptBitSet = function(self, bit)
        if bit == self.PowerDownToggleBit then
            --# Is this a shield toggle?
            if self.PowerDownToggleBit == 0 then
                --# Shield toggles are the opposite
                --# state mapping of other toggles
                --# so switch on shield
                self.On = true
            else
                --# Switch off intel or production
                self.On = false
            end
        end
        
        --# Do base class version after
        baseClassArg.OnScriptBitSet(self, bit)
    end,
    
    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  These functions allow user to toggle whether or not 
    --#*  the ACU's auto-toggling can run on this unit. 
    --#** 
    OnScriptBitClear = function(self, bit)
        if bit == self.PowerDownToggleBit then
            --# Is this a shield toggle?
            if self.PowerDownToggleBit == 0 then
                --# Shield toggles are the opposite
                --# state mapping of other toggles
                --# so switch off shield
                self.On = false
            else
                --# Switch on intel or production
                self.On = true
            end
        end
        
        --# Do base class version after
        baseClassArg.OnScriptBitClear(self, bit)
    end,
    
    
    --#*
    --#*  Gilbot-X says:
    --#*
    --#*  These functions are called when the weapon-toggle button
    --#*  (which I use as the auto-pause toggle button) using bit 1
    --#*  is pressed.  They are called from my override of OnScriptBitSet
    --#*  and OnScriptBitClear defined in my hook of Unit.lua.
    --#**
    OnScriptBit1Clear = function(self)
        --# Notify/unregister with ACU
        self:TellACU_NotToUseAutoToggleOnMe()       
    end,

    OnScriptBit1Set = function(self)
        --# Notify/register with ACU
        self:TellACU_ToUseAutoToggleOnMe()
    end,
    
    
    
    --#*
    --#*  Gilbot-X says:  
    --#* 
    --#*  Had to override this to make sure that tables are 
    --#*  cleaned up when a unit completes an upgrade and 
    --#*  destroys itself.
    --#**
    OnDestroy = function(self)
        
        --# This next block is for debugging only.
        --# This can be delete wheh debugging is done.
        if false and debugAutoToggle then
          LOG('AutoToggle: OnDestroy called on ' .. self.DebugId)
        end
        
        --# Gilbot-X says:  
        --# I added this block of code so that when 
        --# units upgrade they get cleaned out of 
        --# tables in the ACU's auto-toggle tables.
        self:TellACU_NotToUseAutoToggleOnMe()
             
        --# Finally the rest is GPG code.
        baseClassArg.OnDestroy(self)
    end
    
}


return resultClass

end