do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#**  Hook File :  /lua/aeonunits.lua
--#**  Modded By :  Gilbot-X
--#**
--#**  Summary   :  Unit class generic overrides for Aeon faction
--#**
--#**  Note: 310 lines in original file so if you get an error 
--#**  subtract 310 from the line number it gives you to find 
--#**  where it is in this hook file
--#**  
--#****************************************************************************

local ExtendedAdjacencyUnit = 
    import('/mods/gilbotsimprovements/extendedadjacencyunit.lua').ExtendedAdjacencyUnit
local MakeSensitiveShieldUser = 
    import('/mods/gilbotsimprovements/sensitiveshield.lua').MakeSensitiveShieldUser
local HydroCarbonPowerPlant = 
    import('/mods/gilbotsimprovements/hcpp.lua').HydroCarbonPowerPlant
local TimeBasedMassCollectionUnit = 
    import('/mods/gilbotsimprovements/timebasedmassextractor.lua').TimeBasedMassCollectionUnit
local Unit = import('/lua/sim/Unit.lua').Unit

--#* 
--#*  Gilbot-X says:
--#*
--#*  All Aeon defense-weapon structures (such as artillery) extend 
--#*  This class directly from their script files.
--#*
--#** 
AStructureUnit = Class(ExtendedAdjacencyUnit) {}


--#-------------------------------------------------------------
--#  MASS COLLECTION UNITS
--#-------------------------------------------------------------

--#* 
--#*  Gilbot-X says:
--#*
--#*  In the original aeonunits, when AMassCollectionUnit extends 
--#*  MassCollectionUnit it does not add any extra code.
--#*
--#*  In the original defaultunits.lua, when MassCollectionUnit extends StructureUnit
--#*  with some extra code which I have moved into GilbotMassCreationUnit.
--#*
--#*  I've also changed this so Aeon Mexes always pause production when upgrading.
--#**
do --(start of baseclass definition)
local BaseClass = MakeSensitiveShieldUser(TimeBasedMassCollectionUnit)
AMassCollectionUnit = Class(BaseClass) {

    --# This is required.
    ActiveAnimationDoesNotUseInfiniteLoop = true,
    
    --#* 
    --#*  Gilbot-X says:
    --#*
    --#*  This is called once our MeX is built and is ready to become
    --#*  active.  I always do my unit initilisation code here.
    --#**: 
    OnCreate = function(self)
        --# Call base class code first
        BaseClass.OnCreate(self)
        
        --# Get animation values from BP file
        self:GetAnimationSettings()
    end,
    
    
    --# The defaults are the values for the T1 MeX
    --# If the animations were made consistently, 
    --# we wouuldn't need this code.  In fact, 
    --# if I knew how to edit an animation file, 
    --# I would do that instead.  Maybe one day...
    PackedAtAnimationStart = true,
    UpgradesFromPacked = true,
    UnPackedAnimationFraction = 0.125,
    PackedUpAnimationFraction = 0,
    AnimationTurnTime = 0,
    AnimationTurnRate = 0,
    
    --# Get animation values from BP file
    GetAnimationSettings = function(self)
        local settings = self:GetBlueprint().Display.ActiveAnimationSettings
        if settings then 
            self.PackedAtAnimationStart = settings.PackedAtStart or false
            self.UpgradesFromPacked = settings.UpgradesFromPacked or false
            self.UnPackedAnimationFraction = settings.UnPackedFraction or 0
            self.PackedUpAnimationFraction = settings.PackedUpFraction or 0
            self.AnimationTurnTime = settings.TurnTime or 0
            self.AnimationTurnRate = settings.TurnRate or 0
            --# This is set based on animation settings
            self.IsPacked = self.PackedAtAnimationStart
        else
            WARN("AMassCollectionUnit: OnStopBeingBuilt: "..
                 "Could not find ActiveAnimationSettings in Blueprint file!") 
        end
        
    end,
    
    

    

    --#* 
    --#*  Gilbot-X says:
    --#* 
    --#*  This function manipulates the open animation
    --#*  to give the effect of the unit packing/unpacking.
    --#*  This makes it look like the unit can change into a defensive position.
    --#*  Do not change the numerical values in these functions
    --#*  or the animation will lose its smoothness and appear to jump.
    --#*          
    --#*  This should play part of the animation but stop itself 
    --#*  in a position a certain fraction of the way through 
    --#*  where the Mex appears to be packed away.
    --#*  T1 animation starts and finishes packed, but 
    --#*  T2 and T3 animations start and finish in unpacked positions.
    --#*  Note: GPG version had code (that would never be reached) making it go backwards!!  
    --#*  IMO making it go backwards for one (part of) cycle actually looks worse.
    --#*
    --#**
    DoPackUpAnimation = function(self, isPackedRequested)
    
        --# This next line is just a safegaurd because
        --# a couple of cases this was running out of turn after an upgrade!
        --# Note: This code is not in the AutoPackUnit class because different units 
        --# use different animation manipulators to pack and unpack.
        --# The Aeon MeX uses its active animation.  Most units will use their 
        --# 'open' animation instead.       
        if not self.ActiveAnimationManipulator then
            WARN("AMassCollectionUnit: DoPackUpAnimation: Called without ActiveAnimationManipulator!")
            return
        end
        
        --# Units that do the pack up animation will not 
        --# play the active animation in an infinite loop.        
        --self:WaitForActiveAnimationManipulator()
        do--##### Was WaitForActiveAnimationManipulator function
            if self.ActiveAnimationManipulator then
                --local thisThread = CurrentThread()
                --if self.LastThread ~= thisThread then
                --    LOG('AMassCollectionUnit#' .. self:GetEntityId() 
                --    .. ': NewThread=' .. repr(thisThread)
                --    )
                --    self.LastThread = thisThread 
                --end
                --# Wait until we are at beginning of animation.
                WaitFor(self.ActiveAnimationManipulator)
                --# We have reached the end of the animation loop
                self.ActiveAnimationLoopRunning = false
            else
                WARN('AMassCollectionUnit: '
                  .. 'ActiveAnimationManipulator not defined.')
            end
        
            --# The rest is specific to those units that have active 
            --# animations but also animate (it or something else) to pack up.
            --# First, record if we are packed (or not) now that
            --# we have reached the end of another animation loop.
            self.IsPacked = self.PackedAtAnimationStart
            
            --# Next, check/record if we are ready to upgrade.
            if self.IsPacked and self.UpgradesFromPacked then 
                self.NotReadyToUpgrade = false
            end
        end--##### Was WaitForActiveAnimationManipulator function
            
        --# Do we need to pack or unpack to acheive this?
        if isPackedRequested then 
            --# We are deactivating production, so perform pack animation
            --# The three numerical values below must not be changed.
             
            --# Play Part of Animation 
              self.ActiveAnimationManipulator:SetAnimationTime(0)
              self.ActiveAnimationManipulator:SetRate(self.AnimationTurnRate)
              while self.ActiveAnimationManipulator:GetAnimationTime() < self.AnimationTurnTime do
                  WaitTicks(1)
              end
              --# Stop animation at near enough right place
              self.ActiveAnimationManipulator:SetRate(0)
      
              --# Force animation into more exact down position specified in script
              --self.ActiveAnimationManipulator:SetAnimationFraction(self.PackedUpAnimationFraction)
              if self.PackedAtAnimationStart then 
                  self.ActiveAnimationManipulator:SetAnimationFraction(0)
                  self.ActiveAnimationLoopRunning = false
              end
              
        --# We are activating production, so perform unpack animation.
        else
            --# Play Part of Animation 
            local startTime = self.ActiveAnimationManipulator:GetAnimationTime()
            local nowTime = startTime
            local timeDif = 0
            self.ActiveAnimationManipulator:SetRate(-1* self.AnimationTurnRate)
            while timeDif < self.AnimationTurnTime do 
                nowTime = self.ActiveAnimationManipulator:GetAnimationTime()
                timeDif = math.abs(nowTime - startTime)
                WaitTicks(2)
            end
              
            --# Stop animation at near enough right place
            self.ActiveAnimationManipulator:SetRate(0)
            
            --# Force animation into more exact down position specified in script
            --self.ActiveAnimationManipulator:SetAnimationFraction(self.UnPackedAnimationFraction)
            if not self.PackedAtAnimationStart then 
                self.ActiveAnimationManipulator:SetAnimationFraction(self.UnPackedAnimationFraction)
                if self.UnPackedAnimationFraction == 0 then 
                    self.ActiveAnimationLoopRunning = false
                end
            end
        end
    end,

              
    --#*
    --#*  Gilbot-X says:
    --#*        
    --#*  I've overrided this so Aeon Mexes always pause production when upgrading.
    --#**  
    UpgradingState = State {
        Main = function(self)
            Unit.OnProductionPaused(self)
            BaseClass.UpgradingState.Main(self)
        end,

        OnStopBuild = BaseClass.UpgradingState.OnStopBuild,

        OnFailedToBuild = function(self)
            Unit.OnProductionUnpaused(self)
            BaseClass.OnFailedToBuild(self)
        end,
    },
    
}
end --(end of baseclass definition)




--#-------------------------------------------------------------
--#  ENERGY CREATION UNITS
--#-------------------------------------------------------------

--#*
--#*  Gilbot-X says:
--#*  
--#*  AHydrocarbonPowerPlant extends TimeBasedEnergyCreationUnit
--#*  with all the extra GPG code that AEnergyCreationUnit adds to
--#*  extend EnergyCreationUnit.  So we can use AHydrocarbonPowerPlant
--#*  as a base class for our Aeon HCPP alongside any class that 
--#*  extends the StructureUnit class.
--#*
--#*  When EnergyCreationUnit extends StructureUnit it declares
--#*  LandBuiltHiddenBones = {'Floatation'} which it seems is what all units
--#*  that were provisionally designed to be built on water are given.
--#*  That code is now also in TimeBasedEnergyCreationUnit. 
--#*
--#*  In the Aeon case, when AEnergyCreationUnit extended 
--#*  EnergyCreationUnit some extra code was also added to look through the 
--#*  self.AmbientEffects key and add any effects there to the unit.
--#*  That code is now also in TimeBasedEnergyCreationUnit. There was
--#*  also an unused key declared (self.NumUsedAdjacentUnits=0) 
--#*  but I haven't put that in TimeBasedEnergyCreationUnit
--#*  because after searching all lua files I saw it was never used.
--#**
do --(start of baseclass definition)
local BaseClass = MakeSensitiveShieldUser(HydroCarbonPowerPlant)
AHydrocarbonPowerPlant = Class(BaseClass) {}

end --(end of baseclass definition)
end --(of non-destructive hook)