do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#**  Hook File:  /lua/cybranunits.lua
--#**  Modded by:  Gilbot-X
--#**
--#**  Summary  :  Unit class generic overrides for Cybran faction
--#**
--#**  Note: 362 lines in original file so if you get an error 
--#**  subtract 362 from the line number it gives you to find 
--#**  where it is in this hook file.
--#**  
--#****************************************************************************

local ExtendedAdjacencyUnit = 
    import('/mods/gilbotsimprovements/extendedadjacencyunit.lua').ExtendedAdjacencyUnit
local HydroCarbonPowerPlant = 
    import('/mods/gilbotsimprovements/hcpp.lua').HydroCarbonPowerPlant
local TimeBasedMassCollectionUnit = 
    import('/mods/gilbotsimprovements/timebasedmassextractor.lua').TimeBasedMassCollectionUnit
local MakeAutoToggleUnit = 
    import('/mods/gilbotsimprovements/autotoggle.lua').MakeAutoToggleUnit
  
--#* 
--#*  Gilbot-X says:
--#*
--#*  All Cybran defense-weapon structures (such as artillery) extend 
--#*  This class directly from their script files.
--#*
--#** 
CStructureUnit = Class(ExtendedAdjacencyUnit) {}
    
    
--#-------------------------------------------------------------
--#  MASS COLLECTION UNITS
--#-------------------------------------------------------------

--#* 
--#*  Gilbot-X says:
--#*
--#*  In the original cybranunits, when CMassCollectionUnit extends 
--#*  MassCollectionUnit it does not add any extra code.
--#*
--#*  In the original defaultunits.lua, when MassCollectionUnit extends StructureUnit
--#*  with some extra code which I have moved into GilbotMassCreationUnit.
--#*
--#*  Let the stealth toggle be part of ACU's auto-toggle list
--#*  but set the switching off priority to top of the list 
--#*  (after mass fabricators)
--#**
local BaseClass = MakeAutoToggleUnit(TimeBasedMassCollectionUnit, 5, 2)
CMassCollectionUnit = Class(BaseClass) {

    --#*
    --#*  Gilbot-X says:
    --#*      
    --#*  I overrided OnCreate so I could 
    --#*  do non-static variable initialization at this time.
    --#** 
    OnCreate = function(self)
        --# Perform original class version
        BaseClass.OnCreate(self)
        
        --# This is used by code from my (now integrated) 
        --# Maintenance Consumption Breakdown mod.
        --# It uses this table to keep track of how many / 
        --# which types of intel are active.
        self.EnabledResourceDrains = {
            Stealth = false,
            ProductionCosts = true,
        }
    end,
    
    
    --# Gilbot-X: This is called once our MeX is built and is ready to become
    --# active.  I always do my unit initilisation code here.
    OnStopBeingBuilt = function(self, builder, layer)
        --# original version will just SetMaintenanceConsumptionActive
        --# and call its superclass, which itself calls
        --# self:PlayActiveAnimation() and then calls the version in unit.lua
        BaseClass.OnStopBeingBuilt(self, builder, layer)
        
        --# Turn stealth off by default, otherwise it's
        --# annoying turning off stealth manually when building
        --# first base at start of game.        
        self:SetScriptBit('RULEUTC_StealthToggle', true)
    end,
}





--#-------------------------------------------------------------
--#  ENERGY CREATION UNITS
--#-------------------------------------------------------------

--#*
--#*  Gilbot-X says:
--#*  
--#*  CHydrocarbonPowerPlant extends TimeBasedEnergyCreationUnit
--#*  with all the extra GPG code that CEnergyCreationUnit adds to
--#*  extend EnergyCreationUnit.  So we can use CHydrocarbonPowerPlant
--#*  as a base class for our Aeon HCPP alongside any class that 
--#*  extends the StructureUnit class.
--#*
--#*  When EnergyCreationUnit extends StructureUnit it declares
--#*  LandBuiltHiddenBones = {'Floatation'} which it seems is what all units
--#*  that were provisionally designed to be built on water are given.
--#*  That code is now also in TimeBasedEnergyCreationUnit. 
--#*
--#*  In the Cybran case, when CEnergyCreationUnit extended 
--#*  EnergyCreationUnit some extra code was also added to look through the 
--#*  self.AmbientEffects key and add any effects there to the unit.
--#*  That code is now in TimeBasedEnergyCreationUnit.
--#**
CHydrocarbonPowerPlant = Class(MakeAutoToggleUnit(HydroCarbonPowerPlant, 5, 2)) {}




--#*
--#*  Gilbot-X says:
--#*  
--#*  I've had to cut and paste this code here to make sure it extends
--#*  my ExtendedAdjacencyUnit class, as it gets GPG adjacency it has
--#*  to also be an extended adjacency unit.
--#**
CConstructionStructureUnit = Class(CStructureUnit) {   
    OnCreate = function(self)
        --# Structure stuff
        CStructureUnit.OnCreate(self)

        --# Construction stuff   
        self.EffectsBag = {}
        if self:GetBlueprint().General.BuildBones then
            self:SetupBuildBones()
        end

        if self:GetBlueprint().Display.AnimationBuild then
            self.BuildingOpenAnim = self:GetBlueprint().Display.AnimationBuild
        end

        if self.BuildingOpenAnim then
            self.BuildingOpenAnimManip = CreateAnimator(self)
            self.BuildingOpenAnimManip:SetPrecedence(1)
            self.BuildingOpenAnimManip:PlayAnim(self.BuildingOpenAnim, false):SetRate(0)
            if self.BuildArmManipulator then
                self.BuildArmManipulator:Disable()
            end
        end
        self.BuildingUnit = false
    end,
    
    OnStartBuild = function(self, unitBeingBuilt, order )
        local unitid = self:GetBlueprint().General.UpgradesTo
        
        self.UnitBeingBuilt = unitBeingBuilt
        self.UnitBuildOrder = order
        self.BuildingUnit = true
                       
        CStructureUnit.OnStartBuild(self,unitBeingBuilt, order)
    end,
    
    OnStopBeingBuilt = function(self,builder,layer)
        CStructureUnit.OnStopBeingBuilt(self,builder,layer)
        --# If created with F2 on land, then play the transform anim.
        if(self:GetCurrentLayer() == 'Water') then
            self.TerrainLayerTransitionThread = self:ForkThread(self.TransformThread, true)
        end
    end,

    CreateBuildEffects = function( self, unitBeingBuilt, order )
        local buildbots = EffectUtil.SpawnBuildBots( self, unitBeingBuilt, table.getn(self:GetBlueprint().General.BuildBones.BuildEffectBones), self.BuildEffectsBag )
        EffectUtil.CreateCybranEngineerBuildEffects( self, self:GetBlueprint().General.BuildBones.BuildEffectBones, buildbots, self.BuildEffectsBag )
    end,
    
    --# This will only be called if not in StructureUnit's upgrade state
    OnStopBuild = function(self, unitBeingBuilt)
        CStructureUnit.OnStopBuild(self, unitBeingBuilt)

        self.UnitBeingBuilt = nil
        self.UnitBuildOrder = nil

        if self.BuildingOpenAnimManip and self.BuildArmManipulator then
            self.StoppedBuilding = true
        elseif self.BuildingOpenAnimManip then
            self.BuildingOpenAnimManip:SetRate(-1)
        end

        self.BuildingUnit = false
    end,
    
    OnPaused = function(self)
        --# When factory is paused take some action
        self:StopUnitAmbientSound( 'ConstructLoop' )
        CStructureUnit.OnPaused(self)
        if self.BuildingUnit then
            CStructureUnit.StopBuildingEffects(self, self:GetUnitBeingBuilt())
        end    
    end,
    
    OnUnpaused = function(self)
        if self.BuildingUnit then
            self:PlayUnitAmbientSound( 'ConstructLoop' )
            CStructureUnit.StartBuildingEffects(self, self:GetUnitBeingBuilt(), self.UnitBuildOrder)
        end
        CStructureUnit.OnUnpaused(self)
    end,
    
    StartBuildingEffects = function(self, unitBeingBuilt, order)
        CStructureUnit.StartBuildingEffects(self, unitBeingBuilt, order)
    end,
    
    StopBuildingEffects = function(self, unitBeingBuilt)
        CStructureUnit.StopBuildingEffects(self, unitBeingBuilt)
    end,
    
    WaitForBuildAnimation = function(self, enable)
        if self.BuildArmManipulator then
            WaitFor(self.BuildingOpenAnimManip)
            if (enable) then
                self.BuildArmManipulator:Enable()
            end
        end
    end,

    OnPrepareArmToBuild = function(self)
        CStructureUnit.OnPrepareArmToBuild(self)
        if self.BuildingOpenAnimManip then
            self.BuildingOpenAnimManip:SetRate(self:GetBlueprint().Display.AnimationBuildRate or 1)
            if self.BuildArmManipulator then
                self.StoppedBuilding = false
                ForkThread( self.WaitForBuildAnimation, self, true )
            end
        end
    end,

    OnStopBuilderTracking = function(self)
        CStructureUnit.OnStopBuilderTracking(self)

        if self.StoppedBuilding then
            self.StoppedBuilding = false
            self.BuildArmManipulator:Disable()
            self.BuildingOpenAnimManip:SetRate(-(self:GetBlueprint().Display.AnimationBuildRate or 1))
        end
    end,
    
    CheckBuildRestriction = function(self, target_bp)
        if self:CanBuild(target_bp.BlueprintId) then
            return true
        else
            return false
        end
    end,
       
    CreateReclaimEffects = function( self, target )
        EffectUtil.PlayReclaimEffects( 
            self, target, 
            self:GetBlueprint().General.BuildBones.BuildEffectBones or {0,}, 
            self.ReclaimEffectsBag 
        )
    end,
    
    CreateReclaimEndEffects = function( self, target )
        EffectUtil.PlayReclaimEndEffects( self, target )
    end,         
    
    CreateCaptureEffects = function( self, target )
        EffectUtil.PlayCaptureEffects( 
            self, 
            target, 
            self:GetBlueprint().General.BuildBones.BuildEffectBones or {0,}, 
            self.CaptureEffectsBag 
        )
    end,    
}




end --(of non-destructive hook)