do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#**  Hook File:  /lua/defaultunits.lua
--#**
--#**  Modded by:  Gilbot-X
--#**
--#**  Summary  :  Default definitions of units
--#**
--#**              Original file had 1731 lines so if you get an error,
--#**              you can subtract 1731 from the line number they give 
--#**              you to find where the error was in the hook.
--#**
--#****************************************************************************

local ExtendedAdjacencyUnit = 
    import('/mods/gilbotsimprovements/extendedadjacencyunit.lua').ExtendedAdjacencyUnit
local MakeAutoToggleUnit = 
    import('/mods/gilbotsimprovements/autotoggle.lua').MakeAutoToggleUnit
local GPGStructureUnit = 
    import('/mods/gilbotsimprovements/gpgstructureunit.lua').GPGStructureUnit
local ToggleFactoryUnit = 
    import('/mods/gilbotsimprovements/togglefactory.lua').ToggleFactoryUnit


--#################################################################
--##  STRUCTURE UNITS
--#################################################################
StructureUnit = Class(GPGStructureUnit) {}

--#-------------------------------------------------------------
--#  FACTORY  UNITS
--#-------------------------------------------------------------
FactoryUnit = Class(ToggleFactoryUnit) {}
AirFactoryUnit = Class(FactoryUnit) {}
LandFactoryUnit = Class(FactoryUnit) {}

SeaFactoryUnit = Class(FactoryUnit) {
    ----#  Disable the default rocking behavior
    StartRocking = function(self)
    end,

    StopRocking = function(self)
    end,
}
 
QuantumGateUnit = Class(FactoryUnit) {
    --# Added in FA
    OnKilled = function(self, instigator, type, overkillRatio)
        self:StopUnitAmbientSound( 'ActiveLoop' )
        FactoryUnit.OnKilled(self, instigator, type, overkillRatio)
    end,
}


--#-------------------------------------------------------------
--#  AIR STAGING PLATFORMS UNITS
--#-------------------------------------------------------------
AirStagingPlatformUnit = Class(ExtendedAdjacencyUnit) {
    LandBuiltHiddenBones = {'Floatation'},

    OnStopBeingBuilt = function(self,builder,layer)
        ExtendedAdjacencyUnit.OnStopBeingBuilt(self,builder,layer)
        self:SetMaintenanceConsumptionActive()
    end,
}





--#-------------------------------------------------------------
--#
--#          ECONOMY STRUCTURE UNITS
--#
--#-------------------------------------------------------------

--#-------------------------------------------------------------
--#  ENERGY CREATION UNITS
--#-------------------------------------------------------------
EnergyCreationUnit = Class(ExtendedAdjacencyUnit) {
    LandBuiltHiddenBones = {'Floatation'},
    
    --# I moved this here.
    --# This was used in Cybran and Aeon 
    --# EnergyCreationUnit classes.
    --# I've commented it out because none of 
    --# the extending classes use it.. it must be redundant code.
    --[[
    if self.AmbientEffects then
        local army =  self:GetArmy()
        for k, v in EffectTemplate[self.AmbientEffects] do
            CreateAttachedEmitter(self, 0, army, v)
        end
    end
    ]]
}


--#-------------------------------------------------------------
--#  ENERGY STORAGE UNITS
--#-------------------------------------------------------------
EnergyStorageUnit = Class(ExtendedAdjacencyUnit) {
    LandBuiltHiddenBones = {'Floatation'},

    OnStopBeingBuilt = function(self,builder,layer)
        ExtendedAdjacencyUnit.OnStopBeingBuilt(self,builder,layer)
        local aiBrain = GetArmyBrain(self:GetArmy())
        aiBrain:ESRegisterUnitEnergyStorage(self)
        local curEnergy = aiBrain:GetEconomyStoredRatio('ENERGY')
        if curEnergy > 0.11 then
            self:CreateBlinkingLights('Yellow')
        else
            self:CreateBlinkingLights('Red')
        end
    end,

    OnEnergyStorageStateChange = function(self, newState)
        if newState == 'EconLowEnergyStore' then
            self:CreateBlinkingLights('Red')
        elseif newState == 'EconMidEnergyStore' then
            self:CreateBlinkingLights('Yellow')
        elseif newState == 'EconFullEnergyStore' then
            self:CreateBlinkingLights('Green')
        end
    end,

}

--#-------------------------------------------------------------
--#  MASS COLLECTION UNITS
--#-------------------------------------------------------------
--# These are redefined in other files and the four faction files 
--# referenced them from there now.

--#-------------------------------------------------------------
--#  MASS STORAGE UNITS
--#-------------------------------------------------------------
MassStorageUnit = Class(ExtendedAdjacencyUnit) {
    LandBuiltHiddenBones = {'Floatation'},


    OnStopBeingBuilt = function(self,builder,layer)
        ExtendedAdjacencyUnit.OnStopBeingBuilt(self,builder,layer)
        local aiBrain = GetArmyBrain(self:GetArmy())
        aiBrain:ESRegisterUnitMassStorage(self)
        local curMass = aiBrain:GetEconomyStoredRatio('MASS')
        if curMass > 0.11 then
            self:CreateBlinkingLights('Yellow')
        else
            self:CreateBlinkingLights('Red')
        end
    end,


    OnMassStorageStateChange = function(self, newState)
        if newState == 'EconLowMassStore' then
            self:CreateBlinkingLights('Red')
        elseif newState == 'EconMidMassStore' then
            self:CreateBlinkingLights('Yellow')
        elseif newState == 'EconFullMassStore' then
            self:CreateBlinkingLights('Green')
        end
    end,

}






--#-------------------------------------------------------------
--#
--#         AUTO TOGGLE UNITS
--#
--#-------------------------------------------------------------




--#-------------------------------------------------------------
--#  MASS FABRICATION UNITS
--#-------------------------------------------------------------
do --(start of baseclass definition)
local BaseClass = MakeAutoToggleUnit(ExtendedAdjacencyUnit, 4, 1)
MassFabricationUnit = Class(BaseClass) {

    LandBuiltHiddenBones = {'Floatation'},

    OnStopBeingBuilt = function(self,builder,layer)
        BaseClass.OnStopBeingBuilt(self,builder,layer)
        self:SetMaintenanceConsumptionActive()
        self:SetProductionActive(true)
    end,

    OnConsumptionActive = function(self)
        BaseClass.OnConsumptionActive(self)
        self:SetMaintenanceConsumptionActive()
        self:SetProductionActive(true)
    end,

    OnConsumptionInActive = function(self)
        BaseClass.OnConsumptionInActive(self)
        self:SetMaintenanceConsumptionInactive()
        self:SetProductionActive(false)
    end,
    
    --# These were added in FA to add sound effect
    OnProductionPaused = function(self)
        BaseClass.OnProductionPaused(self)
        self:StopUnitAmbientSound( 'ActiveLoop' )
    end,

    OnProductionUnpaused = function(self)
        BaseClass.OnProductionUnpaused(self)
        self:PlayUnitAmbientSound( 'ActiveLoop' )
    end,
      
}
end --(end of baseclass definition)




--#-------------------------------------------------------------
--#  RADAR UNITS
--#-------------------------------------------------------------
do --(start of baseclass definition)
local BaseClass = MakeAutoToggleUnit(ExtendedAdjacencyUnit, 3, 3)
RadarUnit = Class(BaseClass) {

    LandBuiltHiddenBones = {'Floatation'},

    OnStopBeingBuilt = function(self,builder,layer)
        BaseClass.OnStopBeingBuilt(self,builder,layer)
        self:SetMaintenanceConsumptionActive()
    end,
    
    OnIntelDisabled = function(self)
        BaseClass.OnIntelDisabled(self)
        self:DestroyIdleEffects()
        self:DestroyBlinkingLights()
        self:CreateBlinkingLights('Red')
    end,

    OnIntelEnabled = function(self)
        BaseClass.OnIntelEnabled(self)
        self:DestroyBlinkingLights()
        self:CreateBlinkingLights('Green')
        self:CreateIdleEffects()
    end,
}
end --(end of baseclass definition)




--#-------------------------------------------------------------
--#  RADAR JAMMER UNITS
--#    (stealth field towers only)
--#-------------------------------------------------------------
do --(start of baseclass definition)
local BaseClass = MakeAutoToggleUnit(ExtendedAdjacencyUnit, 5, 4)
RadarJammerUnit = Class(BaseClass) {

    LandBuiltHiddenBones = {'Floatation'},

    --#  Shut down intel while upgrading
    OnStartBuild = function(self, unitbuilding, order)
        BaseClass.OnStartBuild(self, unitbuilding, order)
        self:SetMaintenanceConsumptionInactive()
        self:DisableIntel('Jammer')
        self:DisableIntel('RadarStealthField')
    end,

    --#  If we abort the upgrade, re-enable the intel
    OnStopBuild = function(self, unitBeingBuilt)
        BaseClass.OnStopBuild(self, unitBeingBuilt)
        self:SetMaintenanceConsumptionActive()
        self:EnableIntel('Jammer')
        self:EnableIntel('RadarStealthField')
    end,

    --#  If we abort the upgrade, re-enable the intel
    OnFailedToBuild = function(self)
        BaseClass.OnStopBuild(self)
        self:SetMaintenanceConsumptionActive()
        self:EnableIntel('Jammer')
        self:EnableIntel('RadarStealthField')
    end,

    OnStopBeingBuilt = function(self,builder,layer)
        BaseClass.OnStopBeingBuilt(self,builder,layer)
        self:SetMaintenanceConsumptionActive()
    end,
    
    OnIntelEnabled = function(self)
        BaseClass.OnIntelEnabled(self)
        if self.IntelEffects and not self.IntelFxOn then
			self.IntelEffectsBag = {}
			self.CreateTerrainTypeEffects( self, self.IntelEffects, 'FXIdle',  self:GetCurrentLayer(), nil, self.IntelEffectsBag )
			self.IntelFxOn = true
		end
    end,

    OnIntelDisabled = function(self)
        BaseClass.OnIntelDisabled(self)
        EffectUtil.CleanupEffectBag(self,'IntelEffectsBag')
        self.IntelFxOn = false
    end,   
    
}
end --(end of baseclass definition)




--#-------------------------------------------------------------
--#  SONAR UNITS
--#-------------------------------------------------------------
do --(start of baseclass definition)
local BaseClass = MakeAutoToggleUnit(ExtendedAdjacencyUnit, 3, 3)
SonarUnit = Class(BaseClass) {

    LandBuiltHiddenBones = {'Floatation'},

    OnStopBeingBuilt = function(self,builder,layer)
        BaseClass.OnStopBeingBuilt(self,builder,layer)
        self:SetMaintenanceConsumptionActive()
    end,
    
    CreateIdleEffects = function(self)
        BaseClass.CreateIdleEffects(self)
        self.TimedSonarEffectsThread = self:ForkThread( self.TimedIdleSonarEffects )
    end,
    
    TimedIdleSonarEffects = function( self )
        local layer = self:GetCurrentLayer()
        local army = self:GetArmy()
        local pos = self:GetPosition()
        
        if self.TimedSonarTTIdleEffects then
            while not self:IsDead() do
                for kTypeGroup, vTypeGroup in self.TimedSonarTTIdleEffects do
                    local effects = self.GetTerrainTypeEffects( 'FXIdle', layer, pos, vTypeGroup.Type, nil )
                    
                    for kb, vBone in vTypeGroup.Bones do
                        for ke, vEffect in effects do
                            emit = CreateAttachedEmitter(self,vBone,army,vEffect):ScaleEmitter(vTypeGroup.Scale or 1)
                            if vTypeGroup.Offset then
                                emit:OffsetEmitter(vTypeGroup.Offset[1] or 0, vTypeGroup.Offset[2] or 0,vTypeGroup.Offset[3] or 0)
                            end
                        end
                    end                    
                end
                self:PlayUnitSound('Sonar')
                WaitSeconds( 6.0 )                
            end
        end
    end,
    
    DestroyIdleEffects = function(self)
        self.TimedSonarEffectsThread:Destroy()
        BaseClass.DestroyIdleEffects(self)
    end,    
    
    OnIntelDisabled = function(self)
        BaseClass.OnIntelDisabled(self)
        self:DestroyBlinkingLights()
        self:CreateBlinkingLights('Red')
    end,

    OnIntelEnabled = function(self)
        BaseClass.OnIntelEnabled(self)
        self:DestroyBlinkingLights()
        self:CreateBlinkingLights('Green')
    end,
      
}
end --(end of baseclass definition)



--#-------------------------------------------------------------
--#  SHIELD STRUCTURE UNITS
--#-------------------------------------------------------------

--#*
--#*  Gilbot-X says:
--#*
--#*  In the original file, no code is added to the StructureUnit class.
--#*  This code is all for AutoToggle.
--#**
ShieldStructureUnit = MakeAutoToggleUnit(ExtendedAdjacencyUnit, 0, 6)



--#-------------------------------------------------------------
--#  SHIELD HOVER UNITS
--#-------------------------------------------------------------
ShieldHoverLandUnit = MakeAutoToggleUnit(HoverLandUnit, 0, 5)


--#-------------------------------------------------------------
--#  SHIELD LAND UNITS
--#-------------------------------------------------------------
ShieldLandUnit = MakeAutoToggleUnit(LandUnit, 0, 5)


--#-------------------------------------------------------------
--#  SHIELD SEA UNITS
--#-------------------------------------------------------------
ShieldSeaUnit  = MakeAutoToggleUnit(SeaUnit,  0, 5)

end --(of non-destructive hook)