do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#**  Hook File :  /lua/shield.lua
--#**  Modded By :  Gilbot-X
--#**
--#**  Summary   :  Shield for units and structures.
--#**
--#**  Note: 477 lines in original file so if you get an error 
--#**  subtract 477 from the line number it gives you to find 
--#**  where it is in this hook file.
--#**  
--#****************************************************************************


--#*
--#*  Gilbot-X says:  
--#*
--#*  I overrided this class definition so that all shields can use 
--#*  my new shield strength adjacency bonus.
--#**
local PreviousVersion = Shield
Shield = Class(PreviousVersion) {


    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I overrided this to allow X and Z offsets.
    --#*  My pipeline units need those offsets.
    --#**
     OnCreate = function(self,spec)
        self.Trash = TrashBag()
        self.Owner = spec.Owner
        self.MeshBp = spec.Mesh
        self.MeshZBp = spec.MeshZ
        self.ImpactMeshBp = spec.ImpactMesh
        if spec.ImpactEffects ~= '' then
            self.ImpactEffects = EffectTemplate[spec.ImpactEffects]
        else
            self.ImpactEffects = {}
        end

        self:SetSize(spec.Size)
        self:SetMaxHealth(spec.ShieldMaxHealth)
        self:SetHealth(self,spec.ShieldMaxHealth)

        --# Show our 'lifebar'
        self:UpdateShieldRatio(1.0)

        self:SetRechargeTime(spec.ShieldRechargeTime or 5, spec.ShieldEnergyDrainRechargeTime or 5)
        
        --# Gilbot-X:
        --# This next line is changed
        self:SetOffset(spec.ShieldXOffset or 0, 
                       spec.ShieldVerticalOffset, 
                       spec.ShieldZOffset or 0
        )
        
        self:SetVizToFocusPlayer('Always')
        self:SetVizToEnemies('Intel')
        self:SetVizToAllies('Always')
        self:SetVizToNeutrals('Intel')

        self:AttachBoneTo(-1,spec.Owner,-1)

        self:SetShieldRegenRate(spec.ShieldRegenRate)
        self:SetShieldRegenStartTime(spec.ShieldRegenStartTime)

        self.OffHealth = -1
        self.PassOverkillDamage = spec.PassOverkillDamage

        ChangeState(self, self.OnState)
    end,
    
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I added this to allow X and Z offsets.
    --#**
    SetOffset = function(self, x , y, z)
        self.ShieldXOffset = x
        self.ShieldVerticalOffset = y
        self.ShieldZOffset = z
    end,
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I overrided this to allow X and Z offsets.
    --#**
    CreateShieldMesh = function(self)
        self:SetCollisionShape( 'Sphere', 0, 0, 0, self.Size/2)
        self:SetMesh(self.MeshBp)
        --# I changed this next line to allow x and z offsets
        self:SetParentOffset(
          Vector(
            self.ShieldXOffset or 0,
            self.ShieldVerticalOffset,
            self.ShieldZOffset or 0
          )
        )
        self:SetDrawScale(self.Size)

        if self.MeshZ == nil then
            self.MeshZ = Entity { Owner = self.Owner }
            self.MeshZ:SetMesh(self.MeshZBp)
            Warp( self.MeshZ, self.Owner:GetPosition() )
            self.MeshZ:SetDrawScale(self.Size)
            self.MeshZ:AttachBoneTo(-1,self.Owner,-1)
            --# I changed this to allow for x and z offsets
            self.MeshZ:SetParentOffset(
              Vector(
                self.ShieldXOffset,
                self.ShieldVerticalOffset,
                self.ShieldZOffset
              )
            )

            self.MeshZ:SetVizToFocusPlayer('Always')
            self.MeshZ:SetVizToEnemies('Intel')
            self.MeshZ:SetVizToAllies('Always')
            self.MeshZ:SetVizToNeutrals('Intel')
        end
    end,
    
    
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I overrided this to use the shield strength adjacency bonus.
    --#**
    GetHealth = function(self)
        return math.floor((PreviousVersion.GetHealth(self) * (self.Owner.ShieldStrengthMod or 1)))
    end,
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I overrided this to use the shield strength adjacency bonus.
    --#**
    GetMaxHealth = function(self)
        return math.floor((PreviousVersion.GetMaxHealth(self) * (self.Owner.ShieldStrengthMod or 1)))
    end,
    
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I added this so when I increase shield max health
    --#*  as part of an upgrade, the health and regenrate also get updated
    --#**
    SetMaxHealthAndRegenRate = function(self, newMaxHealthWithoutBonus)
        --# First we work out if shield was more or less 
        --# at full health before we change its max health.
        local myShieldHealthWasPrettyMuchFull = false
        local oldHealthWithoutBonus    = PreviousVersion.GetHealth(self) 
        local oldMaxHealthWithoutBonus = PreviousVersion.GetMaxHealth(self)
        local tolerance = 5        
        --# Give a bit of tolerance. So if shield is within 5 HP of maximum health...
        if PreviousVersion.GetHealth(self) >= (oldMaxHealthWithoutBonus-tolerance) then
            myShieldHealthWasPrettyMuchFull = true
        end
        
        --# Now set max health just as original base version does
        PreviousVersion.SetMaxHealth(self, newMaxHealthWithoutBonus)
        
        --# If the shield is not enabled, and its health was almost full, then ...
        if (not self:IsOn()) and myShieldHealthWasPrettyMuchFull then
            --# Set it's health to the new maximum (it stays full).               
            PreviousVersion.SetHealth(self, self, newMaxHealthWithoutBonus)
        
        --# otherwise just add the extra HP points to health
        else
            local extraHP = newMaxHealthWithoutBonus - oldMaxHealthWithoutBonus 
            PreviousVersion.SetHealth(self, self, oldHealthWithoutBonus+extraHP)
        end
        
        --# Change RegenRate so it takes a similar amount of time to regenerate
        --# the shield to full health (while it is on).
        local increaseFactor = newMaxHealthWithoutBonus / oldMaxHealthWithoutBonus
        self.RegenRate = math.floor(self.RegenRate * increaseFactor)
        
        --# Show player new shield health
        if true then self:ShowShieldStrengthMessage() end
    end,
    
    
    --#*
    --#*  Gilbot-X says:  
    --#*
    --#*  I added this so that user can see shield HP and HP max
    --#*  on units where these values change.
    --#**
    ShowShieldStrengthMessage = function(self)
        local shieldStrengthMessage = 
            'Shield HP=' .. repr(self:GetHealth()) ..
          ' Max=' .. repr(self:GetMaxHealth())
        --# Notify user
        self.Owner:FlashMessage(shieldStrengthMessage, 3)
    end,
}
    
    
    
--#*
--#*  Gilbot-X says:  
--#*
--#*  I had to put the rest of shield.lua in this hook file again
--#*  because I've changed a base class that these other classes extend.
--#*  This is all GPG code, I haven't touched it.
--#**
UnitShield = Class(Shield){

    OnCreate = function(self,spec)
        self.Trash = TrashBag()
        self.Owner = spec.Owner
        self.ImpactEffects = EffectTemplate[spec.ImpactEffects]        
        self.CollisionSizeX = spec.CollisionSizeX or 1
        self.CollisionSizeY = spec.CollisionSizeY or 1
        self.CollisionSizeZ = spec.CollisionSizeZ or 1
        self.CollisionCenterX = spec.CollisionCenterX or 0
        self.CollisionCenterY = spec.CollisionCenterY or 0
        self.CollisionCenterZ = spec.CollisionCenterZ or 0
        self.OwnerShieldMesh = spec.OwnerShieldMesh or ''

        self:SetSize(spec.Size)

        self:SetMaxHealth(spec.ShieldMaxHealth)
        self:SetHealth(self,spec.ShieldMaxHealth)

        --# Show our 'lifebar'
        self:UpdateShieldRatio(1.0)
        
        self:SetRechargeTime(spec.ShieldRechargeTime or 5, spec.ShieldEnergyDrainRechargeTime or 5)
        self:SetVerticalOffset(spec.ShieldVerticalOffset)

        self:SetVizToFocusPlayer('Always')
        self:SetVizToEnemies('Intel')
        self:SetVizToAllies('Always')
        self:SetVizToNeutrals('Always')

        self:AttachBoneTo(-1,spec.Owner,-1)

        self:SetShieldRegenRate(spec.ShieldRegenRate)
        self:SetShieldRegenStartTime(spec.ShieldRegenStartTime)

        self.PassOverkillDamage = spec.PassOverkillDamage
        
        --# Changed this so sensitive shields can start off
        --# if key is set in the blueprint.  
        --# Can't use StartOff as that is used for enhnacements
        --# and any shield enhancement would also start switched off.
        if not spec.CreateSwitchedOff then
            ChangeState(self, self.OnState)
        end
    end,

    CreateImpactEffect = function(self, vector)
        local army = self:GetArmy()
        local OffsetLength = Util.GetVectorLength(vector)
        local ImpactEnt = Entity { Owner = self.Owner }

        Warp( ImpactEnt, self:GetPosition())
        ImpactEnt:SetOrientation(OrientFromDir(Vector(-vector.x,-vector.y,-vector.z)),true)

        for k, v in self.ImpactEffects do
            CreateEmitterAtBone( ImpactEnt, -1, army, v ):OffsetEmitter(0,0,OffsetLength)
        end
        WaitSeconds(1)

        ImpactEnt:Destroy()
    end,

    CreateShieldMesh = function(self)
        self:SetCollisionShape( 'Box', self.CollisionCenterX, self.CollisionCenterY, 
            self.CollisionCenterZ, self.CollisionSizeX, self.CollisionSizeY, self.CollisionSizeZ
        )
        self.Owner:SetMesh(self.OwnerShieldMesh,true)
    end,

    RemoveShield = function(self)
        self:SetCollisionShape('None')
        self.Owner:SetMesh(self.Owner:GetBlueprint().Display.MeshBlueprint, true)
    end,

    OnDestroy = function(self)
        if not self.Owner.MyShield or self.Owner.MyShield:GetEntityId() == self:GetEntityId() then
            self.Owner:SetMesh(self.Owner:GetBlueprint().Display.MeshBlueprint, true)
        end
        self:UpdateShieldRatio(0)
        ChangeState(self, self.DeadState)
    end,
    
}



--#*
--#*  Gilbot-X says:  
--#*
--#*  I had to put the rest of shield.lua in this hook file again
--#*  because I've changed a base class that these other classes extend.
--#*  This is all GPG code, I haven't touched it.
--#**
AntiArtilleryShield = Class(Shield){

    OnCollisionCheckWeapon = function(self, firingWeapon)
        local bp = firingWeapon:GetBlueprint()
        if bp.CollideFriendly == false then
            if self:GetArmy() == firingWeapon.unit:GetArmy() then
                return false
            end
        end
        --# Check DNC list
        if bp.DoNotCollideList then
            for k, v in pairs(bp.DoNotCollideList) do
                if EntityCategoryContains(ParseEntityCategory(v), self) then
                        return false
                end
            end
        end           
        if bp.ArtilleryShieldBlocks then
            return true
        end
        return false
    end,

    --# Return true to process this collision, false to ignore it.
    OnCollisionCheck = function(self,other)
        if other:GetArmy() == -1 then
            return false
        end

        if other:GetBlueprint().Physics.CollideFriendlyShield and 
          other.DamageData.ArtilleryShieldBlocks then
            return true
        end
        
        if other.DamageData.ArtilleryShieldBlocks and 
          IsEnemy(self:GetArmy(),other:GetArmy()) then
            return true
        end

        return false
    end,
}


end --(of non-destructive hook)