do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#** Hook File:  /lua/system/blueprints.lua
--#**
--#** Modded By:  Gilbot-X
--#**             Updated for FA on July 14 2008
--#**
--#** Changes:
--#**    Cloak effect code added. 
--#**    Exponential Hydrocarbon Power Plants (HCPP) code added.
--#**    Factory Toggles code added.
--#**    Auto-toggke code added.
--#**    Exponentiona Mass Extractors code added.
--#**   
--#*********************************************************************



--#* 
--#*  Gilbot-X says:
--#* 
--#*  ModBlueprints is called once 
--#*  at the start of every game. 
--#*  This gives us a hook to modify the 
--#*  blueprint values for all units.
--#**
local OldModBlueprints = ModBlueprints
function ModBlueprints(all_blueprints)
    --# Do any previously defined code first
    OldModBlueprints(all_blueprints)
    
    --# This function is defined below
    --# and was part of the 'Transparent Cloak' mod.
    for id,bp in all_blueprints.Unit do
        --# This function is defined below
        --# as part of cloak effect mod
        ExtractCloakMeshBlueprint(bp)
    end
    
    --# This function is defined below
    --# and was part of the 'Exponential HCPP' mod.
    ModHCPPs(all_blueprints.Unit)
    
    --# This function is defined below
    --# and was part of the 'Exponential Mass' mod.
    ModMassExtractors(all_blueprints.Unit)
    
    --# This function is defined below
    --# and was part of the 'Factory Toggles' mod.
    GiveAllTogglesToFactories(all_blueprints.Unit)
    
    --# This function is defined below
    --# and was part of the 'Auto Toggle' mod.
    GiveExtraToggleToAutoToggleUnits(all_blueprints.Unit)
    
    --# This function is defined below
    --# and was part of the 'Auto Pack' mod.
    GiveExtraToggleToAutoPackUnits(all_blueprints.Unit)
    
    --# This function is defined below
    --# and was part of the 'Resource Network' mod.
    GiveDirectFireOverlayToRemotePipelines(all_blueprints.Unit)
    
    --# This function is defined below
    DontShowLifeBarsOnWalls(all_blueprints.Unit)
end
    

--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by me
--#*  I prefer that walls don't show lifebars.
--#*  You can still check the HP of a wall when selecting
--#*  it and looking at the units details panel in the UI.
--#**
function DontShowLifeBarsOnWalls(all_bps) 

    --# Put all the unit blueprints for walls into an array.
    local walls = {
        all_bps['uab5101'], 
        all_bps['ueb5101'], 
        all_bps['urb5101'],
        all_bps['xsb5101'], 
    }
    
    --# Give each factory these 6 toggles.
    for arrayIndex, bp in walls do
        bp.LifeBarRender = false
    end
end




--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by me
--#*  for my mod Factory Toggles.
--#*  Each factory unit gets 6 toggles
--#*  that can be used to set intel/shield etc
--#*  on units coming out of the factory.
--#**
function GiveAllTogglesToFactories(all_bps) 

    --# Put all the unit blueprints for factories into an array.
    local factories = {
        all_bps['uab0101'], all_bps['uab0102'], all_bps['uab0103'], 
        all_bps['uab0201'], all_bps['uab0202'], all_bps['uab0203'],
        all_bps['uab0301'], all_bps['uab0302'], all_bps['uab0303'], all_bps['uab0304'],
        
        all_bps['ueb0101'], all_bps['ueb0102'], all_bps['ueb0103'], 
        all_bps['ueb0201'], all_bps['ueb0202'], all_bps['ueb0203'],
        all_bps['ueb0301'], all_bps['ueb0302'], all_bps['ueb0303'], all_bps['ueb0304'],
        
        all_bps['urb0101'], all_bps['urb0102'], all_bps['urb0103'], 
        all_bps['urb0201'], all_bps['urb0202'], all_bps['urb0203'],
        all_bps['urb0301'], all_bps['urb0302'], all_bps['urb0303'], all_bps['urb0304'],
        
        all_bps['xsb0101'], all_bps['xsb0102'], all_bps['xsb0103'], 
        all_bps['xsb0201'], all_bps['xsb0202'], all_bps['xsb0203'],
        all_bps['xsb0301'], all_bps['xsb0302'], all_bps['xsb0303'], all_bps['xsb0304'],
    }
    
    --# Give each factory these 6 toggles.
    for arrayIndex, bp in factories do
        bp.General.ToggleCaps = {
            RULEUTC_CloakToggle = true,
            RULEUTC_StealthToggle = true,
            RULEUTC_IntelToggle = true,
            RULEUTC_ShieldToggle = true,
            RULEUTC_JammingToggle = true,
            RULEUTC_WeaponToggle = true,
        }
    end
end




--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by me
--#*  for my AutoToggle units.
--#*  The Weapon toggle gets used with a new custom button 
--#*  to toggle ACU auto-toggle on and off.
--#**
function GiveExtraToggleToAutoToggleUnits(all_bps) 

    --# Put all units that autotoggle will work on into an array
    local autoToggleUnits = {
        all_bps['uab1104'], all_bps['uab1303'], --Mass fabs
        all_bps['uab3101'], all_bps['uab3201'], all_bps['uab3104'], -- Radar towers
        all_bps['uab3102'], all_bps['uab3202'], -- Stationary Sonar
        all_bps['uab4203'], -- Stealth Tower
        all_bps['uab4202'], all_bps['uab4301'], -- Shield Generators
        all_bps['ual0307'], -- Mobile Shield Generator
        all_bps['uaa0310'], -- Czar
        all_bps['uac1401b'], -- Shield Strength Enhancer
        
        all_bps['ueb1104'], all_bps['ueb1303'], --Mass fabs
        all_bps['ueb3101'], all_bps['ueb3201'], all_bps['ueb3104'], -- Radar towers
        all_bps['ueb3102'], all_bps['ueb3202'], -- Stationary Sonar
        all_bps['ueb4203'], -- Stealth Tower
        all_bps['ueb4202'], all_bps['ueb4301'], -- Shield Generators
        all_bps['uel0307'], -- Mobile Shield Generator
        all_bps['xes0205'], -- Shield Boat
        
        all_bps['urb1102'], --# HCPP Stealth (cloak ignored?)
        all_bps['urb1103'], all_bps['urb1202'], all_bps['urb1302'], -- Mex Stealth
        all_bps['urb1104'], all_bps['urb1303'], --Mass fabs
        all_bps['urb3101'], all_bps['urb3201'], all_bps['urb3104'], -- Radar towers
        all_bps['urb3102'], all_bps['urb3202'], -- Stationary Sonar
        all_bps['urb4203'], -- Stealth Tower
        all_bps['urb4202'], all_bps['urb4204'], all_bps['urb4205'], -- Shield Generators
        all_bps['urb4206'], all_bps['urb4207'],                     -- More Shield Generators
        all_bps['url0307'], -- Mobile Shield Generator
        all_bps['xrs0205'], -- Stealth boat
        
        all_bps['xsb1104'], all_bps['xsb1303'], --Mass fabs
        all_bps['xsb3101'], all_bps['xsb3201'], all_bps['xsb3104'], -- Radar towers
        all_bps['xsb3102'], all_bps['xsb3202'], -- Stationary Sonar
        all_bps['xsb4203'], -- Stealth Tower
        all_bps['xsb4202'], all_bps['xsb4301'], -- Shield Generators
        all_bps['xsl0307'], -- Mobile Shield Generator
    }
    
    --# Give each factory these 6 toggles.
    for arrayIndex, bp in autoToggleUnits do
        --# Add order buttons
        if not bp.General.ToggleCaps then bp.General.ToggleCaps = {} end
        bp.General.ToggleCaps.RULEUTC_WeaponToggle = true
        --# Use custom butmap for order button
        if not bp.General.OrderOverrides then bp.General.OrderOverrides = {} end
        bp.General.OrderOverrides.RULEUTC_WeaponToggle = {
            bitmapId = 'auto-toggle',
            helpText = 'toggle_auto_toggle',
        }
    end
end




--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by me
--#*  for my AutoPack units.
--#*  The Weapon toggle gets used with a new custom button 
--#*  to toggle autopack on and off.
--#**
function GiveExtraToggleToAutoPackUnits(all_bps) 

    --# Put all units that autotoggle will work on into an array
    local autoPackUnits = {
        --# AEON Sensitive Shield
        all_bps['uab1102'], --# HCPP
        all_bps['uab1103'], all_bps['uab1202'], all_bps['uab1302'], --Mass extractors
 
        --# UEF DLS
        all_bps['ueb1102'], --# HCPP
        all_bps['ueb1103'], all_bps['ueb1202'], all_bps['ueb1302'], --Mass extractors
    }
    
    --# Give each factory these 6 toggles.
    for arrayIndex, bp in autoPackUnits do
        --# Give two order buttons
        if not bp.General.ToggleCaps then bp.General.ToggleCaps = {} end
        bp.General.ToggleCaps.RULEUTC_ProductionToggle = true   
        bp.General.ToggleCaps.RULEUTC_WeaponToggle = true
        --# Override the button on the weapon toggle to give it the AP button
        if not bp.General.OrderOverrides then bp.General.OrderOverrides = {} end
        bp.General.OrderOverrides.RULEUTC_WeaponToggle = {
            bitmapId = 'auto-pack',
            helpText = 'toggle_auto_pack',
        }
    end
end



--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by me for my remote Pipeline units.
--#*  The direct fire weapon overlay allowsplayers to see the range 
--#*  of remote adjacency for the selected pipeline. 
--#**
function GiveDirectFireOverlayToRemotePipelines(all_bps) 

    --# Put all units that autotoggle will work on into an array
    local remotePipelineUnits = {
        all_bps['gab5201'], all_bps['gab5301'], 
        all_bps['geb5201'], all_bps['geb5301'], 
        all_bps['grb5201'], all_bps['grb5301'], all_bps['grb5301b'],
    }

    --# Give each remote pipeline a direct fire weapon
    for arrayIndex, bp in remotePipelineUnits do
        bp.Weapon[1].RangeCategory = 'UWRC_DirectFire'
        bp.Weapon[1].WeaponCategory = 'Direct Fire'
        table.insert(bp.Categories, 'OVERLAYDIRECTFIRE')
        bp.General.SelectionPriority = 4
    end
end


--#* 
--#*  Gilbot-X says:
--#* 
--#*  This was written by Covert Jaguar 
--#*  for his mod Transparent Cloak Effect.
--#*  Each unit gets a new blueprint mesh table 
--#*  (name has a _cloak suffix) that points to
--#*  a semitransparent mesh normally used for 
--#*  formation previews.
--#**
function ExtractCloakMeshBlueprint(bp)
    local meshid = bp.Display.MeshBlueprint
    if not meshid then return end

    local meshbp = original_blueprints.Mesh[meshid]
    if not meshbp then return end

    local shadername = 'UnitFormationPreview'

    local cloakmeshbp = table.deepcopy(meshbp)
    if cloakmeshbp.LODs then
        for i,lod in cloakmeshbp.LODs do
            lod.ShaderName = shadername
        end
    end
    cloakmeshbp.BlueprintId = meshid .. '_cloak'
    bp.Display.CloakMeshBlueprint = cloakmeshbp.BlueprintId
    MeshBlueprint(cloakmeshbp)
end



    
--#* 
--#*  Gilbot-X says:
--#* 
--#*  Bonuses on hydrocarbon plant increase over time.
--#*  Makes holding hydrocarbon plant more important and
--#*  makes game more territorial.
--#** 
function ModHCPPs(all_bps)     
    --# To haved the same values as GPG blueprints,
    --# use these settings:
    --# T1Start = 100,
    --# ..otherwise use your own to get the balance you like.
    --#
    local modSettings = {
        --# T1
        T1Start = 100,
        T1Cap = 5000,
        --# All GrowthConstants: These are constants in exponential formula.
        T1GrowthConstant = 1.02,
        --T1GrowthConstant = 1.05 -- use this one for testing
        

    }
    
    local hydrocarbonunits = {
        all_bps['uab1102'], 
        all_bps['ueb1102'],
        all_bps['urb1102'],
        all_bps['xsb1102'],
    }
    
    for id,bp in hydrocarbonunits do
        --# Add OutputModifier to BP according to tech level
        if not bp.Economy.EnergyProductionGrowthConstant then 
            bp.Economy.EnergyProductionGrowthConstant = modSettings.T1GrowthConstant 
        end
        --# Check user has not set anything in BP themselves
        if not bp.Economy.MaxProductionPerSecondEnergy then
            --# Blueprints ony need one of AutoUpgradeAt or MaxProductionPerSecondEnergy
            bp.Economy.MaxProductionPerSecondEnergy = modSettings.T1Cap  
        end
    end
    
end
        
        

--#*
--#*  Gilbot-X says:
--#*
--#*  Mass production of mass extractors increases over time.
--#*  Makes holding mass extractors more important relative to 
--#*  mass fabricators and makes game more territorial.
--#** 
function ModMassExtractors(all_bps) 

--[[

To have the same values as GPG blueprints, use these settings:
    T1Start = 2,
    T2Start = 6,
    T3Start = 12,
..otherwise use your own to get the balance you like.
  
Giving them all the same start value stops the production jumping when a unit upgrades because the production is based on base production level * factor.  Problem is, newly built T2 and T3 MeX become the same as a T1!!  Therefore I use BuildOnlyTech1 stop them from being built directly - you have to upgrade to get them!  In effect those units just lend their appearance to mark the level of production so the enemy has some idea.  There is code in my MassCollectionUnit class in the defaultunits hook that just doesn't pass on the % increase if the jump in starting values will cause a crazy jump in production, in which case the % increase is lost and the T2 made from an old T1 has the same production as a freshly built T2.  Its a bit of a context-sensitive safeguard in that it guesses whether or not you intended for the % increase to be passed on.

]]

    local modSettings = {
        AutoUpgradeTech1 = true,
        AutoUpgradeTech2 = true,
        BuildOnlyTech1 = false,
        --# T1
        T1Start = 2,
        T1CanUpgradeFrom = 2, --# can upgrade straight away
        T1Cap = 12,
        --# T2
        T2Start = 12,
        T2CanUpgradeFrom = 15, --# NB: setting to false means no manual upgrade possible 
        T2Cap = 30,
        --# T3
        T3Start = 30,
        T3Cap = 100,
        --# All GrowthConstants: These are constants in exponential formula.
        T1GrowthConstant = 1.05,
        T2GrowthConstant = 1.03,
        T3GrowthConstant = 1.02,
    }
    
    local mexunits = {
        TECH1 = {all_bps['uab1103'], all_bps['ueb1103'], all_bps['urb1103'], all_bps['xsb1103'],},
        TECH2 = {all_bps['uab1202'], all_bps['ueb1202'], all_bps['urb1202'], all_bps['xsb1202'],},
        TECH3 = {all_bps['uab1302'], all_bps['ueb1302'], all_bps['urb1302'], all_bps['xsb1302'],},
    }
    
    for unusedArrayIndex, bp in mexunits.TECH1 do
        --# This next line allows us to restrict upgrades
        --# by (perhaps conditionally) removing the upgrade from the build menu
        bp.Economy.BuildableCategory = {
            'BUILTBYTECH1MASSFAB' .. string.upper(bp.General.FactionName),
        }

        --# Add MassProductionGrowthConstant to BP according to tech level
        if not bp.Economy.MassProductionGrowthConstant then
            bp.Economy.MassProductionGrowthConstant = modSettings.T1GrowthConstant 
        end
        --# Set AllowManualUpgradeAt key in BP according to tech level
        if not bp.Economy.AllowManualUpgradeAt then 
            bp.Economy.AllowManualUpgradeAt = modSettings.T1CanUpgradeFrom 
        end
        
        --# Check user has not set anything in BP themselves
        if not bp.Economy.AutoUpgradeAt and 
           not bp.Economy.MaxProductionPerSecondMass then
            --# Blueprints ony need one of AutoUpgradeAt or MaxProductionPerSecondMass
            if modSettings.AutoUpgradeTech1 then
                bp.Economy.AutoUpgradeAt = modSettings.T1Cap 
            else
                bp.Economy.MaxProductionPerSecondMass = modSettings.T1Cap  
            end
        end
    end
        
    for unusedArrayIndex, bp in mexunits.TECH2 do
        --# These next 2 line allows us to restrict upgrades
        --# by (perhaps conditionally) removing the upgrade from the build menu
        table.insert(bp.Categories, 'BUILTBYTECH1MASSFAB' .. string.upper(bp.General.FactionName))
        bp.Economy.BuildableCategory = {
            'BUILTBYTECH2MASSFAB' .. string.upper(bp.General.FactionName) ,
        }
        --# Add MassProductionGrowthConstant to BP according to tech level
        if not bp.Economy.MassProductionGrowthConstant then
            bp.Economy.MassProductionGrowthConstant = modSettings.T2GrowthConstant 
        end
        --# Set AllowManualUpgradeAt key in BP according to tech level
        if not bp.Economy.AllowManualUpgradeAt then 
            bp.Economy.AllowManualUpgradeAt = modSettings.T2CanUpgradeFrom 
        end
        
        --# Check user has not set anything in BP themselves
        if not bp.Economy.AutoUpgradeAt and 
           not bp.Economy.MaxProductionPerSecondMass then
            --# Blueprints ony need one of AutoUpgradeAt or MaxProductionPerSecondMass
            if modSettings.AutoUpgradeTech2 then
                bp.Economy.AutoUpgradeAt = modSettings.T2Cap  
            else
                bp.Economy.MaxProductionPerSecondMass = modSettings.T2Cap  
            end
        end
        
        if modSettings.BuildOnlyTech1 then
            bp.Economy.ProductionPerSecondMass = modSettings.T1Start
            RemoveBuiltByCategories(bp)
        else
            bp.Economy.ProductionPerSecondMass = modSettings.T2Start
        end
    end
    
        
    for unusedArrayIndex, bp in mexunits.TECH3 do
        --# This next line allows us to restrict upgrades
        --# by (perhaps conditionally) removing the upgrade from the build menu
        table.insert(bp.Categories, 'BUILTBYTECH2MASSFAB' .. string.upper(bp.General.FactionName))
        --# Add MassProductionGrowthConstant to BP according to tech level
        if not bp.Economy.MassProductionGrowthConstant then
            bp.Economy.MassProductionGrowthConstant = modSettings.T3GrowthConstant 
        end
        --# T3 is highest level so put cap on instead of autoupgrade
        if not bp.Economy.MaxProductionPerSecondMass then 
            bp.Economy.MaxProductionPerSecondMass = modSettings.T3Cap  
        end
        --# See comments on similar block of code above
        if modSettings.BuildOnlyTech1 then
            bp.Economy.ProductionPerSecondMass = modSettings.T1Start
            RemoveBuiltByCategories(bp)
        else
            bp.Economy.ProductionPerSecondMass = modSettings.T3Start
        end
    end
       
   
    
    --# Rename inconsistently named blueprint key 
    --# that points to animation files
    for techId, techGroup in mexunits do
        for unusedArrayIndex, bp in techGroup do
            local faction = string.upper(bp.General.FactionName)
            if faction == "AEON" then
                --# This is for the auto-defend ability
                bp.General.ToggleCaps.RULEUTC_ShieldToggle = false  
                
            elseif faction == "UEF" then
                --# Rename inconsistently named blueprint key 
                --# that points to animation files
                bp.Display.AnimationActivate = bp.Display.AnimationOpen
                
            elseif faction == "CYBRAN" then
                --# Rename inconsistently named blueprint key 
                --# that points to animation files
                bp.Display.AnimationActivate = bp.Display.AnimationOpen
            end
        end
    end
    
end
    
    
--#*
--#*  Gilbot-X says:
--#*
--#*  This is called by ModMassExtractors above
--#*  to stop T2 and T3 Mass extractors from being built
--#*  (that's a mod option).
--#**    
function RemoveBuiltByCategories(bp) 
    local newCategories = {}
    for k,v in bp.Categories do
        if not (
          string.find(v, 'BUILTBY') 
          and (
              string.find(v, 'ENGINEER') 
           or string.find(v, 'COMMANDER')
          ) 
        )              
        then
            table.insert(newCategories, v)
        else 
            --LOG("Exponential Mass mod: Removed cat " .. v)
        end
    end
    bp.Categories= newCategories
end
    
    
    
    
end --(end of non-destructive hook)