do --(start of non-destructive hook)
--#****************************************************************************
--#**
--#**  Hook File :  /lua/terrannunits.lua
--#**  Modded By :  Gilbot-X
--#**
--#**  Summary   :  Unit class generic overrides for UEF faction
--#**
--#**  Note: 397 lines in original file so if you get an error 
--#**  subtract 397 from the line number it gives you to find 
--#**  where it is in this hook file
--#**  
--#****************************************************************************

local ExtendedAdjacencyUnit = 
    import('/mods/gilbotsimprovements/extendedadjacencyunit.lua').ExtendedAdjacencyUnit
local MakeDamageLimitationSystemUser = 
    import('/mods/gilbotsimprovements/dls.lua').MakeDamageLimitationSystemUser
local HydroCarbonPowerPlant = 
    import('/mods/gilbotsimprovements/hcpp.lua').HydroCarbonPowerPlant
local TimeBasedMassCollectionUnit = 
    import('/mods/gilbotsimprovements/timebasedmassextractor.lua').TimeBasedMassCollectionUnit
    
--#* 
--#*  Gilbot-X says:
--#*
--#*  All UEF defense-weapon structures (such as artillery) extend 
--#*  This class directly from their script files.
--#*
--#** 
TStructureUnit = Class(ExtendedAdjacencyUnit) {}

--#-------------------------------------------------------------
--#  MASS COLLECTION UNITS
--#-------------------------------------------------------------

--#* 
--#*  Gilbot-X says:
--#*
--#*  In the original cybranunits, when TMassCollectionUnit extends 
--#*  MassCollectionUnit it does not add any extra code.
--#*
--#*  In the original defaultunits.lua, when MassCollectionUnit extends StructureUnit
--#*  with some extra code which I have moved into GilbotMassCreationUnit.
--#**
local UEFMassExtractorsUseDLS = true
if UEFMassExtractorsUseDLS then 
    local BaseClass = MakeDamageLimitationSystemUser(TimeBasedMassCollectionUnit)
    TMassCollectionUnit = Class(BaseClass) {}
else
    TMassCollectionUnit = Class(TimeBasedMassCollectionUnit) {}
end

    
--#-------------------------------------------------------------
--# ENERGY CREATION STRUCTURES
--#-------------------------------------------------------------

--#*
--#*  Gilbot-X says:
--#*  
--#*  THydrocarbonPowerPlant extends TimeBasedEnergyCreationUnit
--#*  with all the extra GPG code that TEnergyCreationUnit adds to
--#*  extend StructureUnit.  So we can use THydrocarbonPowerPlant
--#*  as a base class for our Aeon HCPP alongside any class that 
--#*  extends the StructureUnit class.
--#*
--#*  When EnergyCreationUnit extends StructureUnit it declares
--#*  LandBuiltHiddenBones = {'Floatation'} which it seems is what all units
--#*  that were provisionally designed to be built on water are given.
--#*  That code is now also in TimeBasedEnergyCreationUnit. 
--#*
--#*  In the UEF case, there was no extra code added 
--#*  when TEnergyCreationUnit extended EnergyCreationUnit.
--#**
do --(start of baseclass definition)
  local BaseClass = MakeDamageLimitationSystemUser(HydroCarbonPowerPlant)
  THydrocarbonPowerPlant = Class(BaseClass) {}
end --(end of baseclass definition)




--#*
--#*  Gilbot-X says:
--#*  
--#*  I've had to cut and paste this code here to make sure it extends
--#*  my ExtendedAdjacencyUnit class, as it gets GPG adjacency it has
--#*  to also be an extended adjacency unit.
--#**
TPodTowerUnit = Class(TStructureUnit) {
    OnStopBeingBuilt = function(self, builder, layer)
        TStructureUnit.OnStopBeingBuilt(self, builder, layer)
        ChangeState( self, self.FinishedBeingBuilt )
    end,
    
    PodTransfer = function(self, pod, podData)
        --# Set the pod as active, set new parent and creator for the pod, store the pod handle
        if not self.PodData[pod.PodName].Active then
            if not self.PodData then
                self.PodData = {}
            end
            self.PodData[pod.PodName] = table.deepcopy( podData )
            self.PodData[pod.PodName].PodHandle = pod
            pod:SetParent(self, pod.PodName)
        end
    end,
    
    OnCaptured = function(self, captor)
        --# Iterate through pod data and set up callbacks for transfer of pods.
        --# We never get the handle to the new tower, so we set up a new unit capture trigger to do the same thing
        --# not the most efficient thing ever but it makes for never having to update the capture codepath here
        for k,v in self.PodData do
            if v.Active then
                v.Active = false
            
                --# store off the pod name so we can give to new unit
                local podName = k
                local newPod = import('/lua/ScenarioFramework.lua').GiveUnitToArmy( v.PodHandle, captor:GetArmy() )
                newPod.PodName = podName
                
                --# create a callback for when the unit is flipped.  set creator for the new pod to the new tower
                self:AddUnitCallback(
                    function(newUnit, captor)
                        newUnit:PodTransfer( newPod, v )
                    end,
                    'OnCapturedNewUnit'
                )
            end
        end
        
        --# Calling the parent OnCaptured will cause all the callbacks to happen and happiness will reign !
        TStructureUnit.OnCaptured(self, captor)
    end,
    
    OnDestroy = function(self)
        TStructureUnit.OnDestroy(self)
        --# Iterate through pod data, kill all the pods and set them inactive
        if self.PodData then
            for k,v in self.PodData do
                if v.Active and not v.PodHandle:IsDead() then
                    v.PodHandle:Kill()
                end
            end
        end
    end,
    
    OnStartBuild = function(self, unitBeingBuilt, order )
        TStructureUnit.OnStartBuild(self,unitBeingBuilt,order)
        local unitid = self:GetBlueprint().General.UpgradesTo
        if unitBeingBuilt:GetUnitId() == unitid and order == 'Upgrade' then
            self.NowUpgrading = true
            ChangeState( self, self.UpgradingState )
        end
    end,
    
    NotifyOfPodDeath = function(self,podName)
        self.PodData[podName].Active = false
    end,
    
    NotifyOfPodStartBuild = function(self)
        if not self.OpeningAnimationStarted then
            self.OpeningAnimationStarted = true
            local bp = self:GetBlueprint()
            if not bp.Display.AnimationOpen then return end    
            if not self.OpenAnim then
                self.OpenAnim = CreateAnimator(self)
                self.Trash:Add(self.OpenAnim)
            end
            self.OpenAnim:PlayAnim(bp.Display.AnimationOpen, false):SetRate(2.0)
            WaitSeconds(0.5)
            if not self.NowUpgrading then
                self.OpenAnim:SetRate(0)
            end
        end    
    end,
    
    NotifyOfPodStopBuild = function(self)
        if self.OpeningAnimationStarted then
            local bp = self:GetBlueprint()
            if not bp.Display.AnimationOpen then return end
            if not self.OpenAnim then return end
            self.OpenAnim:SetRate(1.5)
            self.OpeningAnimationStarted = false
        end    
    end,
    
    SetPodConsumptionRebuildRate = function(self, podData)
        local bp = self:GetBlueprint()
        --# Get build rate of tower
        local buildRate = bp.Economy.BuildRate
        
        local energy_rate = ( podData.BuildCostEnergy / podData.BuildTime ) * buildRate
        local mass_rate = ( podData.BuildCostMass / podData.BuildTime ) * buildRate
        
        --# Set Consumption - Buff system will replace this here
        self:SetConsumptionPerSecondEnergy(energy_rate)
        self:SetConsumptionPerSecondMass(mass_rate)
        self:SetConsumptionActive(true)
    end,
    
    CreatePod = function(self, podName)
        local location = self:GetPosition( self.PodData[podName].PodAttachpoint )
        self.PodData[podName].PodHandle = CreateUnitHPR(self.PodData[podName].PodUnitID, self:GetArmy(), location[1], location[2], location[3], 0, 0, 0)
        self.PodData[podName].PodHandle:SetParent(self, podName)
        self.PodData[podName].Active = true
    end,
    
    OnTransportAttach = function(self, bone, attachee)
        attachee:SetDoNotTarget(true)
        TStructureUnit.OnTransportAttach(self, bone, attachee)
    end,
    
    OnTransportDetach = function(self, bone, attachee)
        attachee:SetDoNotTarget(false)
        TStructureUnit.OnTransportDetach(self, bone, attachee)
    end,
    
    FinishedBeingBuilt = State {
        Main = function(self)
            --# Wait one tick to make sure this wasn't 
            --# captured and we don't create an extra pod
            WaitSeconds(0.1)
            
            --# Create the pod for the kennel.  DO NOT ADD TO TRASH.
            --# This pod may have to be passed to another unit 
            --# after it upgrades.  We cannot let the trash clean it up
            --# when this unit is destroyed at the tail end of the upgrade.  
            --# Make sure the unit dies properly elsewhere.

            self.TowerCaptured = nil
            local bp = self:GetBlueprint()
            for k,v in bp.Economy.EngineeringPods do
                if v.CreateWithUnit and not self.PodData[v.PodName].Active then
                    if not self.PodData then
                        self.PodData = {}
                    end
                    self.PodData[v.PodName] = table.deepcopy( v )
                    self:CreatePod( v.PodName )
                end
            end

            ChangeState( self, self.MaintainPodsState )
        end,
    },

    MaintainPodsState = State {
        Main = function(self)
            self.MaintainState = true
            if self.Rebuilding then
                self:SetPodConsumptionRebuildRate( self.PodData[ self.Rebuilding ] )
                ChangeState( self, self.RebuildingPodState )
            end
            local bp = self:GetBlueprint()
            while true and not self.Rebuilding do
                for k,v in bp.Economy.EngineeringPods do
                    --# Check if all the pods are active
                    if not self.PodData[v.PodName].Active then
                        --# Cost of new pod
                        local podBP = self:GetAIBrain():GetUnitBlueprint( v.PodUnitID )
                        self.PodData[v.PodName].EnergyRemain = podBP.Economy.BuildCostEnergy
                        self.PodData[v.PodName].MassRemain = podBP.Economy.BuildCostMass

                        self.PodData[v.PodName].BuildCostEnergy = podBP.Economy.BuildCostEnergy
                        self.PodData[v.PodName].BuildCostMass = podBP.Economy.BuildCostMass
                        
                        self.PodData[v.PodName].BuildTime = podBP.Economy.BuildTime
                        
                        --# Enable consumption for the rebuilding
                        self:SetPodConsumptionRebuildRate(self.PodData[v.PodName])
                       
                        --# Change to RebuildingPodState
                        self.Rebuilding = v.PodName
                        self:SetWorkProgress(0.01)
                        ChangeState( self, self.RebuildingPodState )
                    end
                end
                WaitSeconds(1)
            end
        end,

        OnProductionPaused = function(self)
            ChangeState( self, self.PausedState )
        end,
    },
    
    RebuildingPodState = State {
        Main = function(self)
            local rebuildFinished = false
            local podData = self.PodData[ self.Rebuilding ]
            repeat
                WaitTicks(1)
                --# While the pod being built isn't finished
                --# Update mass and energy given to new pod - update build bar
                local fraction = self:GetResourceConsumed()
                local energy = self:GetConsumptionPerSecondEnergy() * fraction * 0.1
                local mass = self:GetConsumptionPerSecondMass() * fraction * 0.1
                
                self.PodData[ self.Rebuilding ].EnergyRemain = self.PodData[ self.Rebuilding ].EnergyRemain - energy
                self.PodData[ self.Rebuilding ].MassRemain = self.PodData[ self.Rebuilding ].MassRemain - mass
                
                self:SetWorkProgress( ( self.PodData[ self.Rebuilding ].BuildCostMass - self.PodData[ self.Rebuilding ].MassRemain ) / self.PodData[ self.Rebuilding ].BuildCostMass )
                
                if ( self.PodData[ self.Rebuilding ].EnergyRemain <= 0 ) and ( self.PodData[ self.Rebuilding ].MassRemain <= 0 ) then
                    rebuildFinished = true
                end
            until rebuildFinished
            
            --# create pod, deactivate consumption, clear building
            self:CreatePod( self.Rebuilding )
            self.Rebuilding = false
            self:SetWorkProgress(0)
            self:SetConsumptionPerSecondEnergy(0)
            self:SetConsumptionPerSecondMass(0)
            self:SetConsumptionActive(false)
            
            ChangeState( self, self.MaintainPodsState )
        end,
        
        OnProductionPaused = function(self)
            self:SetConsumptionPerSecondEnergy(0)
            self:SetConsumptionPerSecondMass(0)
            self:SetConsumptionActive(false)
            ChangeState( self, self.PausedState )
        end,
    },
    
    PausedState = State {
        Main = function(self)
            self.MaintainState = false
        end,

        OnProductionUnpaused = function(self)
            ChangeState( self, self.MaintainPodsState )
        end,
    },

    UpgradingState = State {
        Main = function(self)
            self:StopRocking()
            local bp = self:GetBlueprint().Display
            self:DestroyTarmac()
            self:PlayUnitSound('UpgradeStart')
            self:DisableDefaultToggleCaps()
            if bp.AnimationUpgrade then
                local unitBuilding = self.UnitBeingBuilt
                self.AnimatorUpgradeManip = CreateAnimator(self)
                self.Trash:Add(self.AnimatorUpgradeManip)
                local fractionOfComplete = 0
                self:StartUpgradeEffects(unitBuilding)
                self.AnimatorUpgradeManip:PlayAnim(bp.AnimationUpgrade, false):SetRate(0)

                while fractionOfComplete < 1 and not self:IsDead() do
                    fractionOfComplete = unitBuilding:GetFractionComplete()
                    self.AnimatorUpgradeManip:SetAnimationFraction(fractionOfComplete)
                    WaitTicks(1)
                end
                if not self:IsDead() then
                    self.AnimatorUpgradeManip:SetRate(1)
                end
            end
        end,

        OnProductionPaused = function(self)
            self.MaintainState = false
        end,
        
        OnProductionUnpaused = function(self)
            self.MaintainState = true
        end,
        
        OnStopBuild = function(self, unitBuilding)
            TStructureUnit.OnStopBuild(self, unitBuilding)
            self:EnableDefaultToggleCaps()
            if unitBuilding:GetFractionComplete() == 1 then
                NotifyUpgrade(self, unitBuilding)
                self:StopUpgradeEffects(unitBuilding)
                self:PlayUnitSound('UpgradeEnd')
                --# Iterate through pod data and transfer pods to the new unit
                for k,v in self.PodData do
                    if v.Active then
                        unitBuilding:PodTransfer(v.PodHandle, v)
                        v.Active = false
                    end
                end
                self:Destroy()
            end
        end,

        OnFailedToBuild = function(self)
            TStructureUnit.OnFailedToBuild(self)
            self:EnableDefaultToggleCaps()
            self.AnimatorUpgradeManip:Destroy()
            if self:GetCurrentLayer() == 'Water' then
                self:StartRocking()
            end
            self:PlayUnitSound('UpgradeFailed')
            self:PlayActiveAnimation()
            self:CreateTarmac(true, true, true, self.TarmacBag.Orientation, self.TarmacBag.CurrentBP)
            if self.MaintainState then
                ChangeState(self, self.MaintainPodsState)
            else
                ChangeState(self, self.PausedState)
            end
        end,
        
    },
}

end --(of non-destructive hook)