--#****************************************************************************
--#**
--#**  Hook File:  /lua/ui/game/gamemain.lua
--#**
--#**  Modded By:  Gilbot-X, with link to code from Goom
--#**
--#**  Summary  :  Overrided to add keymappings for slider controls
--#**              and autotoggle priority order changes
--#**
--#****************************************************************************

--# Functions in other files to be called by keymappings
local StatSliderMenu = 
    import('/mods/gilbotsimprovements/statslidermenu.lua').StatSliderMenu
    
--# Use this reference to check if a slider menu is already active
--# so we don't show a new one till the previous one is gone.
local ReferenceToLastSliderMenu

--# This effects what is output to the log.
--# Switch it to false if you are not debugging.
local debugKeyMaps = true



--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that a new slider menu can be launched 
--#*  by pressing Alt-R (but only if there isn't one active already).
--#**
CreateStatSliderMenu = function()
    
    if ReferenceToLastSliderMenu and ReferenceToLastSliderMenu.IsMenuActive then
        --# Menu already active when Alt-R pressed,
        --# so do nothing, as we can't dispaly new controls
        --# until previous set are gone.
        return
    end
    
    local unitsSelected = GetSelectedUnits()
    if unitsSelected and type(unitsSelected) == 'table' then 
        ReferenceToLastSliderMenu = StatSliderMenu(unitsSelected)
        --# Don't have to add this menu object to the trash because 
        --# it destroys itself when the unit selection 
        --# changes or the player hits the OK button.
    end
  
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that auto-toggle priorities 
--#*  can be changed by selecting auto-toggler units 
--#*  and using + and - on the keyboard numberpad. 
--#**
ChangeAutoTogglePriority = function(isIncrease)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('AutoToggle: Gamemain.lua: ChangeAutoTogglePriority: ' ..
          'Function called with arg isIncrease=' .. repr(isIncrease)
      )
    end
        
    --# Find out what units are selected
    local unitsSelected = GetSelectedUnits()
    
    --# If exactly one unit is selected
    if unitsSelected 
    and (type(unitsSelected) == 'table') 
    and (table.getsize(unitsSelected) == 1)
    then 
        --# Make a sim call to change its priority
        local functionName = 'DecreaseAutoTogglePriority'
    
        --# Shield size has its own callback function
        --# because the size change is animated
        if isIncrease then
            functionName = 'IncreaseAutoTogglePriority'
        end

        --# Get entity Id to pass as argument
        local entityArg = unitsSelected[1]:GetEntityId()
        
        --# Invoke sim side code.
        SimCallback( 
          {  
            Func=functionName,
            Args={ 
              SelectedUnitEntityId= entityArg,
            }
          }
        )
    end
    
end



--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that auto-toggle priorities 
--#*  can be displayed on all auto-toggle units 
--#*  by pressing CTRL-* on the keyboard numberpad. 
--#*  Make sure that Num Lock is active on your keyboard.
--#**
ToggleAutoTogglePriorityDisplay = function()
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('AutoToggle: Gamemain.lua: ToggleAutoTogglePriorityDisplay: ' ..
          'Function called'
      )
    end
        
    --# Find out what army did this
    local myArmyId = GetFocusArmy()
    
    --# If exactly one unit is selected
    if myArmyId then 
        --# Invoke sim side code.
        SimCallback( 
          {  
            Func='ToggleAutoTogglePriorityDisplay',
            Args={ 
                ArmyId= myArmyId
            }
          }
        )
    end
    
end




--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that network IDs of units 
--#*  in Resource Networks can be displayed by pressing
--#*  CTRL-/ on the keyboard numberpad. 
--#*  Make sure that Num Lock is active on your keyboard.
--#**
ToggleNetworkDisplay = function()
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('Adjacency: Gamemain.lua: ToggleNetworkDisplay: ' ..
          'Function called'
      )
    end
        
    --# Find out what army did this
    local myArmyId = GetFocusArmy()
    
    --# If we have an army
    if myArmyId then 
        --# Invoke sim side code.
        SimCallback( 
          {  
            Func='ToggleNetworkDisplay',
            Args={ 
                ArmyId= myArmyId
            }
          }
        )
    end
    
end



--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that entity IDs 
--#*  can be displayed by selecting units 
--#*  and using CTRL-ALT-/ on the keyboard numberpad. 
--#*  Make sure that Num Lock is active on your keyboard.
--#**
ToggleEntityDisplay = function()
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('Adjacency: Gamemain.lua: ToggleEntityDisplay: ' ..
          'Function called'
      )
    end
        
    local unitsSelected = GetSelectedUnits()
   --# If at least one unit is selected
    if unitsSelected 
    and (type(unitsSelected) == 'table') then
    
        --# Put entity Ids of the units selected
        --# into a new table because the UserUnit references
        --# themselves cannot be marshalled and sent 
        --# to the Sim side.
        local unitEntityIdListArg = {} 
        for unusedArrayIndex, vUnit in unitsSelected do
            table.insert(unitEntityIdListArg, vUnit:GetEntityId())
        end
        
        --# If any units were selected ...
        if (table.getsize(unitsSelected) > 0) then
            --# Invoke sim side code on the unit list.
            SimCallback( 
              {  
                Func='ToggleEntityDisplay',
                Args={ 
                    UnitEntityIdList = unitEntityIdListArg,
                }
              }
            )
        end
    end
    
end



--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that entity IDs 
--#*  can be displayed by selecting units 
--#*  and using CTRL-ALT-S on the keyboard numberpad. 
--#*  Make sure that Num Lock is active on your keyboard.
--#**
ToggleShieldStrengthDisplay = function()
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('Adjacency: Gamemain.lua: ToggleShieldStrengthDisplay: ' ..
          'Function called'
      )
    end
        
    local unitsSelected = GetSelectedUnits()
   --# If at least one unit is selected
    if unitsSelected 
    and (type(unitsSelected) == 'table') then
    
        --# Put entity Ids of the units selected
        --# into a new table because the UserUnit references
        --# themselves cannot be marshalled and sent 
        --# to the Sim side.
        local unitEntityIdListArg = {} 
        for unusedArrayIndex, vUnit in unitsSelected do
            table.insert(unitEntityIdListArg, vUnit:GetEntityId())
        end
        
        --# If any units were selected ...
        if (table.getsize(unitsSelected) > 0) then
            --# Invoke sim side code on the unit list.
            SimCallback( 
              {  
                Func='ToggleShieldStrengthDisplay',
                Args={ 
                    UnitEntityIdList = unitEntityIdListArg,
                }
              }
            )
        end
    end
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I created this so that entity IDs 
--#*  can be displayed by selecting units 
--#*  and using CTRL-ALT-S on the keyboard numberpad. 
--#*  Make sure that Num Lock is active on your keyboard.
--#**
ToggleRateOfFireDisplay = function()
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugKeyMaps then 
      LOG('Adjacency: Gamemain.lua: ToggleRateOfFireDisplay: ' ..
          'Function called'
      )
    end
        
    local unitsSelected = GetSelectedUnits()
   --# If at least one unit is selected
    if unitsSelected 
    and (type(unitsSelected) == 'table') then
    
        --# Put entity Ids of the units selected
        --# into a new table because the UserUnit references
        --# themselves cannot be marshalled and sent 
        --# to the Sim side.
        local unitEntityIdListArg = {} 
        for unusedArrayIndex, vUnit in unitsSelected do
            table.insert(unitEntityIdListArg, vUnit:GetEntityId())
        end
        
        --# If any units were selected ...
        if (table.getsize(unitsSelected) > 0) then
            --# Invoke sim side code on the unit list.
            SimCallback( 
              {  
                Func='ToggleRateOfFireDisplay',
                Args={ 
                    UnitEntityIdList = unitEntityIdListArg,
                }
              }
            )
        end
    end
end


do --(begin non-destructive hook)
--#*
--#*
--#*  Gilbot-X says:
--#*
--#*  I overrided this so that my slider menu can be launched 
--#*  with the ALT-R keymapping.
--#*
--#**
local originalCreateUI = CreateUI
function CreateUI(isReplay)
    originalCreateUI(isReplay)

    --# This next line is for Goom's Area Commands mod.
    import('/mods/gilbotsimprovements/areacommands.lua').Init()
    
    local gilbotsKeyMap = {
        ['Alt-R']  = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").CreateStatSliderMenu()', 
            stringkey = 'create_slider', 
            category = '<LOC keymap_category_0036>Orders', 
            order = 29,
        },
        ['Ctrl-NumMinus']  = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ChangeAutoTogglePriority(false)', 
            stringkey = 'decrease_autotoggle_priority', 
            category = '<LOC keymap_category_0036>Orders', 
            order = 30,
        },
        ['Ctrl-NumPlus']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ChangeAutoTogglePriority(true)', 
            stringkey = 'increase_autotoggle_priority', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 31,
        },
        ['Ctrl-NumStar']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ToggleAutoTogglePriorityDisplay()', 
            stringkey = 'show_autotoggle_priorities', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 32,
        },
        ['Ctrl-NumSlash']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ToggleNetworkDisplay()', 
            stringkey = 'show_network_id', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 33,
        },
        ['Ctrl-Alt-NumSlash']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ToggleEntityDisplay()', 
            stringkey = 'show_entity_id', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 34,
        },
        ['Alt-S']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ToggleShieldStrengthDisplay()', 
            stringkey = 'show_shield_strength', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 35,
        },
        ['Alt-F']   = {
            action =  'UI_Lua import("/lua/ui/game/gamemain.lua").ToggleRateOfFireDisplay()', 
            stringkey = 'show_rate_of_fire', 
            category = '<LOC keymap_category_0036>Orders',  
            order = 36,
        },
    }
    
    IN_AddKeyMapTable(gilbotsKeyMap)
end


--#*
--#*
--#*  Gilbot-X says:
--#*
--#*  I overrided this so that any slider menu gets destroyed 
--#*  if the selection is changed. 
--#*
--#**
local originalOnSelectionChanged = OnSelectionChanged
function OnSelectionChanged(oldSelection, newSelection, added, removed)
    
    if ReferenceToLastSliderMenu then 
        if ReferenceToLastSliderMenu.IsMenuActive then
            --# Destroy menu and make sure nothing can reference it
            ReferenceToLastSliderMenu:Destroy()
            ReferenceToLastSliderMenu = nil
        end
    end

    originalOnSelectionChanged(oldSelection, newSelection, added, removed)
end



end --(end of non-destructive hook)