--#*****************************************************************************
--#* New File : /mods/gilbotsimprovements/masscollectionunit.lua
--#*
--#* Modded By: Gilbot-X
--#*
--#* Summary  : This is used to change GPG's MassCollectionUnit class.
--#*            I put all the code for that class here, with my additions.
--#*            The extra code is code from my Exponential Mass Extractors mod.
--#*            It also allows for upgrade to be blocked until certain 
--#*            conditions are met.
--#*            
--#*
--#*****************************************************************************

--# This is needed by GPG code in this class.
local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')

--# This is a base class we can use when we 
--# need to call base class versions of code.
local ExtendedAdjacencyUnit = 
    import('/mods/gilbotsimprovements/extendedadjacencyunit.lua').ExtendedAdjacencyUnit
local MakeActiveAnimationUnit = 
    import('/mods/gilbotsimprovements/activeanimationunit.lua').MakeActiveAnimationUnit
local BaseClass = MakeActiveAnimationUnit(ExtendedAdjacencyUnit)  

GilbotMassCollectionUnit = Class(BaseClass) {    

    --# From GPG code
    LandBuiltHiddenBones = {'Floatation'},
    
    
    
    --#*    
    --#*  Gilbot-X says:    
    --#* 
    --#*  This was in GPG version and I haven't changed the original code.
    --#**
    OnCreate = function(self)
        BaseClass.OnCreate(self)
    
        --# This is all GPG code.
        local markers = ScenarioUtils.GetMarkers()
        local unitPosition = self:GetPosition()

        for k, v in pairs(markers) do
            if(v.type == 'MASS') then
                local massPosition = v.position
                if  ( (massPosition[1] < unitPosition[1] + 1) and 
                      (massPosition[1] > unitPosition[1] - 1) and
                      (massPosition[2] < unitPosition[2] + 1) and 
                      (massPosition[2] > unitPosition[2] - 1) and
                      (massPosition[3] < unitPosition[3] + 1) and 
                      (massPosition[3] > unitPosition[3] - 1)
                    ) 
                then
                    self:SetProductionPerSecondMass(
                        self:GetProductionPerSecondMass() * (v.amount / 100)
                    )
                    break
                end
            end
        end
    end,
    
    --# Altered in FA
    OnStartBuild = function(self, unitbuilding, order)
        BaseClass.OnStartBuild(self, unitbuilding, order)
        self:AddCommandCap('RULEUCC_Stop')
        local massConsumption = self:GetConsumptionPerSecondMass()
        local massProduction = self:GetProductionPerSecondMass()
        self.UpgradeWatcher = self:ForkThread(self.WatchUpgradeConsumption, massConsumption, massProduction)
    end,

    --# Altered in FA
    OnStopBuild = function(self, unitbuilding, order)
        BaseClass.OnStopBuild(self, unitbuilding, order)
        self:RemoveCommandCap('RULEUCC_Stop')
        if self.UpgradeWatcher then
            KillThread(self.UpgradeWatcher)
            self:SetConsumptionPerSecondMass(0)
            self:SetProductionPerSecondMass(self:GetBlueprint().Economy.ProductionPerSecondMass or 0)                  
        end  
    end,
    
    --# New for FA
    --# band-aid on lack of multiple separate resource requests per unit...  
    --# if mass econ is depleted, take all the mass generated and use it for the upgrade
    WatchUpgradeConsumption = function(self, massConsumption, massProduction)
        while true do
            local fraction = self:GetResourceConsumed()
            --# if we're not getting our full request of energy and mass met...
            if fraction ~= 1 then
               --#check to see if our aiBrain has energy but no mass
               local aiBrain = self:GetAIBrain()
               if aiBrain then
                   local massStored = aiBrain:GetEconomyStored( 'MASS' )
                   if massStored <= 1 then
                       self:SetConsumptionPerSecondMass(massConsumption - massProduction)
                       self:SetProductionPerSecondMass(0)
                   end
               end  
            elseif not self:IsPaused() then
               self:SetConsumptionPerSecondMass(massConsumption)
               self:SetProductionPerSecondMass(massProduction)
            end
            WaitSeconds(0.2)
        end
    end, 
    
    
    --# These were added in FA to add a sound effect
    OnProductionPaused = function(self)
        BaseClass.OnProductionPaused(self)
        self:StopUnitAmbientSound( 'ActiveLoop' )
    end,

    OnProductionUnpaused = function(self)
        BaseClass.OnProductionUnpaused(self)
        self:PlayUnitAmbientSound( 'ActiveLoop' )
    end,

}