--#****************************************************************************
--#**
--#**  New File :  /mods/gilbotsimprovements/utils.lua
--#**  Author(s):  Gilbot-X
--#**
--#**  Summary  :  
--#**      Utilities for manipulating tables used in many of my scripts.
--#**      These are alternatives to versions by GPG which I made 
--#**      because I wanted different behaviour.
--#** 
--#****************************************************************************


--#*
--#*  Gilbot-X says:
--#*      
--#*  Commanders can store information for the army
--#*  and be referenced throughout their lifetime.
--#*  If you are playing non-assassin mode and you 
--#*  lose your commander, then tough, you lose your 
--#*  army's data!
--#*
--#*  If you are playing with Victory conditions set to 
--#*  Supremacy, Annihilation or Sandbox, then whatever is
--#*  built first after the ACU does will get its entity id.
--#*  This function returns nil if that has happened.
--#** 
GetCommanderFromArmyId = function(myArmyId)
    
    --# In patch 3260, 1048576 was the multiple for each army.
    --# these numbers get reused, but once you lose the commander 
    --# you can't build another one, so does its entity ID get used
    --# by another unit when you are not playing Assassin mode?
    local myCommanderEntityId = (myArmyId -1) * 1048576
      --LOG('Adjacency mod: GetMyCommander: Commander entityID for ' 
      --.. ' Army= ' .. repr(myArmyId)
      --.. ' is ' .. repr(myCommanderEntityId)
      --)        
    local myCommander = GetUnitById(myCommanderEntityId)
    
    --# For testing and debugging.  Leave this in for a game 
    --# not in Assassin mode and build something after your 
    --# commander is killed and call this function.
    if not myCommander then 
          --LOG('Adjacency mod: GetMyCommander: Tried to get Commander at UNIT ' 
          --      .. ' entity Id=' .. repr(myCommanderEntityId) 
          --      .. ' but nothing was returned.  Maybe this Commander was killed?'
          --)
        return nil
    elseif not (EntityCategoryContains(categories.COMMAND, myCommander)) then
          --LOG('Adjacency mod: GetMyCommander: Tried to get Commander at UNIT '
          --      .. 'entity Id=' .. repr(myCommanderEntityId) 
          --      .. ' but something else was returned: '
          --      .. ' UnitId=' .. repr(myCommander:GetUnitId())
          --)
        return nil
    else
        return myCommander
    end
end




--#*
--#*  Gilbot-X says:
--#*
--#*  I added this to mimick the function table.remove it but returns false
--#*  if table is empty and doesn't leave any nil keys in the table.
--#*  It also works on tables with string keys as well as arrays.
--#**
function RemoveFromTableByKey(tableArg, keyArg, optionalLeaveEmptyTable)
    --# Do this for safety because when ACU dies, we get problems.
    if type(tableArg) == 'table' then 
        --# Try to remove the item from the table
        local result = {}
        local containsSomething = false
        for k,v in tableArg do
            if k ~= keyArg then
                result[k] = v
                containsSomething = true
            end
        end
        --# Return result
        if containsSomething then return result end
    end
    --# We get here if table passed was invalid
    --# or we end up with an empty table as a result.
    --# Give response according to preference of calling code.     
    if optionalLeaveEmptyTable then 
        return {}
    else 
        return false 
    end
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I added this to mimick the function table.remove it but returns false
--#*  if table is empty and doesn't leave any nil keys in the table.
--#*  It also works on tables with string keys as well as arrays.
--#**
function RemoveFromTableByKeys(tableArg, keyListArg, optionalLeaveEmptyTable)
    --# Do this for safety because when ACU dies, we get problems.
    if type(tableArg) == 'table' and type(keyListArg) == 'table' then 
        --# Try to remove the item from the table
        local result = {}
        local containsSomething = false
        for kTableKey, vTableEntry in tableArg do
            local keyMatched = false
            for kUnusedArrayIndex, vKeyInListArg in keyListArg do
                if kTableKey == vKeyInListArg then keyMatched = true end
            end
            if keyMatched then
                result[kTableKey] = vTableEntry
                containsSomething = true
            end
        end
        --# Return result
        if containsSomething then return result end
    end
    --# We get here if table passed was invalid
    --# or we end up with an empty table as a result.
    --# Give response according to preference of calling code.     
    if optionalLeaveEmptyTable then 
        return {}
    else 
        return false 
    end
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function to the file as a helper function
--#*  It mimicks a function table.removeByValue but returns false
--#*  if table is empty and doesn't leave any nil keys in the table.
--#*  It is designed to deal with lists, so it uses table[k] = v to
--#*  populate the new list.
--#**
function RemoveFromListByValue(tableArg, valueArg, optionalLeaveEmptyTable)
    --# Do this for safety because when ACU dies, we get problems.
    if type(tableArg) == 'table' then 
        --# Try to remove the item from the table
        local result = {}
        local containsSomething = false
        for k,v in tableArg do
            if v ~= valueArg then
                result[k] = v
                containsSomething = true
            end
        end
        --# Return result
        if containsSomething then return result end
    end
    --# We get here if table passed was invalid
    --# or we end up with an empty table as a result.
    --# Give response according to preference of calling code.     
    if optionalLeaveEmptyTable then 
        return {}
    else 
        return false 
    end
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function to the file as a helper function
--#*  It mimicks a function table.removeByValue but returns false
--#*  if table is empty and doesn't leave any nil keys in the table.
--#*  It is designed to deal with arrays, so it uses table.insert to
--#*  populate the new list.
--#**
function RemoveFromArrayByValue(tableArg, valueArg, optionalLeaveEmptyTable)
    --# Do this for safety because when ACU dies, we get problems.
    if type(tableArg) == 'table' then 
        --# Try to remove the item from the table
        local result = {}
        local containsSomething = false
        for k,v in tableArg do
            if v ~= valueArg then
                table.insert(result, v)
                containsSomething = true
            end
        end
        --# Return result
        if containsSomething then return result end
    end
    --# We get here if table passed was invalid
    --# or we end up with an empty table as a result.
    --# Give response according to preference of calling code.     
    if optionalLeaveEmptyTable then 
        return {}
    else 
        return false 
    end
end



--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function to the file as a helper function
--#*  It mimicks a function table.find but returns false
--#*  if table is empty and doesn't return bad values
--#**
function IsValueInTable(tableArg, valueArg)
    --# If it's not a table, return false
    if type(tableArg) ~= 'table' then return false end
    --# return true when u find it
    for k,v in tableArg do
        if v == valueArg then
            return true
        end
    end
    --# false if never found
    return false 
end



--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function to the file as a helper function
--#*  It mimicks a function table.find but works on 2D nested tables
--#*  and returns false if table is empty.  
--#*  Unlike the GPG version, it never returns nil.
--#**
function IsValueIn2DTable(table2DArg, valueArg)
    if type(table2DArg) ~= 'table' then return false end
    --# return true when u find it
    for k1, vTable in table2DArg do
        if type(vTable) == 'table' then 
            for k2, vPotentialMatch in vTable do
                if vPotentialMatch == valueArg then
                    return true
                end
            end
        end
    end
    --# false if never found
    return false
end
    
    
    
    

--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function so I could traverse 
--#*  through a table with string keys in reverse order
--#*  If table is empty it returns false.
--#**
function GetTableKeysReversed(tableArg)
    --# If it's not a table, return false
    if type(tableArg) ~= 'table' then return false end
    --# Put the result in here
    local result = {}
    --# Take each key as we go through the table
    for key, value in tableArg do
        --# and insert it at the front of the result list
        table.insert(result, 0, key)
    end
    return result
end


--#*
--#*  Gilbot-X says:
--#*
--#*  I added this as a new function so I could traverse 
--#*  through an array table with numerical keys in reverse order
--#*  If table is empty it returns false.
--#**
function GetArrayReversed(tableArg)
    --# If it's not a table, return false
    if type(tableArg) ~= 'table' then return false end
    --# Put the result in here
    local result = {}
    --# Take each key as we go through the table
    for key, value in tableArg do
        --# and insert it at the front of the result list
        table.insert(result, 0, value)
    end
    return result
end