--#****************************************************************************
--#**
--#**  New File :  /mods/gilbotsimprovements/autotogglecallbacks.lua
--#**
--#**  Author(s):  Gilbot-X
--#**
--#**  Summary  :  
--#**
--#**  This module contains the Sim-side lua functions that can be invoked
--#**  from the user side.  These need to validate all arguments against
--#**  cheats and exploits.
--#**
--#**  We store the callbacks in a sub-table (instead of directly in the
--#**  module) so that we don't include any.
--#**  
--#***************************************************************************

local GilbotUtils = import('/mods/gilbotsimprovements/utils.lua')

--# This effects what is output to the log.
--# Switch it to false if you are not debugging.
local debugAutoToggle = false


--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the unit selected, checks if 
--#*  it is an AutoToggle unit, and if it is, it 
--#*  calls the ACU function to change its priority position 
--#*  in the ordered priority list.
--#**
function DecreasePriorityCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugAutoToggle then 
      LOG('AutoToggle: AutotoggleCallbacks.lua: ' 
      .. ' DecreasePriorityCallback: ' 
      .. ' Function called with arg data.SelectedUnitEntityId=' 
      .. repr(data.SelectedUnitEntityId)
      )
    end
    
    --# Try to get an autotoggle unit from the ID passed by UI side.
    local selectedUnit = GetEntityById(data.SelectedUnitEntityId)
   
    --# Only pass on call if this is an autotoggle unit.
    if selectedUnit.IsAutoToggleUnit then
        selectedUnit:TellACU_DecreaseMyPriority()
    end
end



--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the unit selected, checks if 
--#*  it is an AutoToggle unit, and if it is, it 
--#*  calls the ACU function to change its priority position 
--#*  in the ordered priority list.
--#**
function IncreasePriorityCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugAutoToggle then 
      LOG('AutoToggle: AutotoggleCallbacks.lua: ' 
      .. ' IncreasePriorityCallback: ' 
      .. ' Function called with arg data.SelectedUnitEntityId=' 
      .. repr(data.SelectedUnitEntityId)
      )
    end
    
    --# Try to get an autotoggle unit from the ID passed by UI side.
    local selectedUnit = GetEntityById(data.SelectedUnitEntityId)
       
    --# Only pass on call if this is an autotoggle unit.
    if selectedUnit.IsAutoToggleUnit then
        selectedUnit:TellACU_IncreaseMyPriority()
    end
  
end



--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the ACU for the focus army,
--#*  checks if it is alive, and if it is, it 
--#*  calls the ACU function to toggle display 
--#   of all priority position numbers 
--#*  in the ordered priority list.
--#**
function TogglePriorityDisplayCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugAutoToggle then 
      LOG('AutoToggle: AutotoggleCallbacks.lua: ' 
      .. ' TogglePriorityDisplayCallback: ' 
      .. ' Function called with arg data.ArmyId=' 
      .. repr(data.ArmyId)
      )
    end
    
    --# Try to get an autotoggle unit from the ID passed by UI side.
    local myCommander = GilbotUtils.GetCommanderFromArmyId(data.ArmyId)
       
    --# Only pass on call if this is an autotoggle unit.
    if myCommander then
        myCommander:ReceiveACUMessage_ToggleDisplayOfAutoToggleNumbers()
    end
  
end
