--#****************************************************************************
--#**
--#**  New File :  /mods/gilbotsimprovements/debuggingcallbacks.lua
--#**
--#**  Author(s):  Gilbot-X
--#**
--#**  Summary  :  
--#**
--#**  This module contains the Sim-side lua functions that can be invoked
--#**  from the user side.  These need to validate all arguments against
--#**  cheats and exploits.
--#**
--#**  We store the callbacks in a sub-table (instead of directly in the
--#**  module) so that we don't include any.
--#**  
--#***************************************************************************


--# This effects what is output to the log.
--# Switch it to false if you are not debugging.
local debugThisFile= false


--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the units selected, checks if 
--#*  it is alive and a ... unit, and if it is, it 
--#*  calls the function to flash a message to the user
--#*  (using the custom naming system).
--#**
function ToggleEntityDisplayCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugThisFile then 
      LOG('DebuggingCallbacks.lua: ' 
      .. ' ToggleEntityDisplayCallback: ' 
      .. ' Function called with ' 
      .. repr(table.getsize(data.UnitEntityIdList))
      .. ' selected units.'
      )
    end
    
    --# For each unit that was selected
    for k, unitEntityId in data.UnitEntityIdList do
        
        --# Try to get an autotoggle unit from the ID passed by UI side.
        local selectedUnit = GetEntityById(unitEntityId)
       
        --# Only pass on call if this is unit is not dead.
        if not (selectedUnit:BeenDestroyed() or selectedUnit:IsDead()) then
            --# FlashMessage is defined in my hook of Unit.lua
            selectedUnit:FlashMessage('e' .. unitEntityId, 5)
        end
    end
end



--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the units selected, checks if 
--#*  it is alive and a ... unit, and if it is, it 
--#*  calls the function to flash a message to the user
--#*  (using the custom naming system).
--#**
function ToggleShieldStrengthDisplayCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugThisFile then 
      LOG('DebuggingCallbacks.lua: ' 
      .. ' ToggleShieldStrengthDisplayCallback: ' 
      .. ' Function called with ' 
      .. repr(table.getsize(data.UnitEntityIdList))
      .. ' selected units.'
      )
    end
    
    --# For each unit that was selected
    for k, unitEntityId in data.UnitEntityIdList do
        
        --# Try to get an autotoggle unit from the ID passed by UI side.
        local selectedUnit = GetEntityById(unitEntityId)
       
        --# Only pass on call if this is unit is not dead.
        if not (selectedUnit:BeenDestroyed() or selectedUnit:IsDead()) then
            --# ShowShieldStrengthMessage is defined in my hook of shield.lua
            --# Show player new shield health
            if selectedUnit.MyShield then selectedUnit.MyShield:ShowShieldStrengthMessage() end
        end
    end
end



--#*
--#*  Gilbot-X says:  
--#*
--#*  This function gets the units selected, checks if 
--#*  it is alive and a ... unit, and if it is, it 
--#*  calls the function to flash a message to the user
--#*  (using the custom naming system).
--#**
function ToggleRateOfFireDisplayCallback(data)
    
    --# This next block is for debugging only.
    --# This can be delete wheh debugging is done.
    if debugThisFile then 
      LOG('DebuggingCallbacks.lua: ' 
      .. ' ToggleRateOfFireDisplayCallback: ' 
      .. ' Function called with ' 
      .. repr(table.getsize(data.UnitEntityIdList))
      .. ' selected units.'
      )
    end
    
    --# For each unit that was selected
    for k, unitEntityId in data.UnitEntityIdList do
        
        --# Try to get an autotoggle unit from the ID passed by UI side.
        local selectedUnit = GetEntityById(unitEntityId)
       
        --# Only pass on call if this is unit is not dead.
        if not (selectedUnit:BeenDestroyed() or selectedUnit:IsDead()) then
            --# ShowRateOfFireMessage is defined in my hook of Unit.lua
            --# Show player the selected units' rate of fire
            if selectedUnit:GetWeaponCount() > 0 then  
                selectedUnit:ShowRateOfFireMessage() 
            end
        end
    end
end
        